# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging
import oslo_messaging

CONF = cfg.CONF
LOG = logging.getLogger(__name__)

notifier_opts = [
    cfg.HostAddressOpt('glare_publisher_id', default="artifact.localhost",
                       help='Default publisher_id for outgoing '
                            'Glare notifications.')]
CONF.register_opts(notifier_opts)


def get_transport():
    return oslo_messaging.get_notification_transport(CONF)


def set_defaults(control_exchange='glare'):
    oslo_messaging.set_transport_defaults(control_exchange)


class Notifier(object):
    """Simple interface to receive Glare notifier."""

    SERVICE_NAME = 'artifact'
    GLARE_NOTIFIER = None

    @classmethod
    def _get_notifier(cls):
        if cls.GLARE_NOTIFIER is None:
            cls.GLARE_NOTIFIER = oslo_messaging.Notifier(
                get_transport(),
                publisher_id=CONF.glare_publisher_id)
        return cls.GLARE_NOTIFIER

    @classmethod
    def notify(cls, context, event_type, body, level='INFO'):
        """Notify Glare listeners with some useful info.

        :param context: User request context
        :param event_type: type of event
        :param body: notification payload
        :param level: notification level ("INFO", "WARN", "ERROR", etc)
        """
        af_notifier = cls._get_notifier()
        method = getattr(af_notifier, level.lower())
        if hasattr(body, 'to_notification'):
            body = body.to_notification()
        method({}, "%s.%s" % (cls.SERVICE_NAME, event_type), body)
        LOG.debug('Notification event %(event)s send successfully for '
                  'request %(request)s', {'event': event_type,
                                          'request': context.request_id})



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import os
import tempfile

from six import BytesIO

from glare.common import exception as exc
from glare.common import store_api
from glare.tests.unit import base


class TestStoreAPI(base.BaseTestArtifactAPI):

    def test_read_data(self):
        """Read data from file, http and database."""

        # test local temp file
        tfd, path = tempfile.mkstemp()
        try:
            os.write(tfd, b'a' * 1000)
            flobj = store_api.load_from_store(
                "file://" + path,
                self.req.context
            )
            self.assertEqual(b'a' * 1000, store_api.read_data(flobj))
            flobj = store_api.load_from_store(
                "file://" + path,
                self.req.context
            )
            self.assertRaises(exc.RequestEntityTooLarge,
                              store_api.read_data, flobj, limit=999)
        finally:
            os.remove(path)

        # test sql object
        values = {'name': 'ttt', 'version': '1.0'}
        self.sample_artifact = self.controller.create(
            self.req, 'sample_artifact', values)
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'a' * 100), 'application/octet-stream')
        flobj = self.controller.download_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob')
        self.assertEqual(b'a' * 100, store_api.read_data(flobj['data']))
        flobj = self.controller.download_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob')
        self.assertRaises(exc.RequestEntityTooLarge,
                          store_api.read_data, flobj['data'], limit=99)

        # test external http
        flobj = store_api.load_from_store(
            'https://www.apache.org/licenses/LICENSE-2.0.txt',
            self.req.context
        )
        self.assertEqual(3967, len(store_api.read_data(flobj)))
        flobj = store_api.load_from_store(
            'https://www.apache.org/licenses/LICENSE-2.0.txt',
            self.req.context
        )
        self.assertRaises(exc.RequestEntityTooLarge,
                          store_api.read_data, flobj, limit=3966)



# Copyright 2017 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class BaseStoreAPI(object):

    def add_to_backend(self, context, blob_id, data, verifier=None):
        """Save data to database store type and return location info

        :param blob_id: id of artifact
        :param data: file iterator
        :param context: user context
        :param verifier:signature verified
        :return: database location uri
        """
        raise NotImplementedError()

    def get_from_store(self, uri, context):
        """Load file from database store

        :param uri: blob uri
        :param context: user context
        :return: file iterator
        """
        raise NotImplementedError()

    def delete_from_store(self, uri, context):
        """Delete blob from database store

        :param uri: blob uri
        :param context: user context
        """
        raise NotImplementedError()



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from functools import partial
import inspect

import mock
from oslo_config import cfg
from oslo_log import log as logging

from glare.api.middleware import fault
from glare.common import exception as exc
from glare.tests.unit import base

CONF = cfg.CONF
logging.register_options(CONF)


class TestFaultMiddleware(base.BaseTestCase):

    @staticmethod
    def get_response(value=None, exception=Exception):
        if value is None:
            raise exception
        return value

    def _build_middleware(self):
        return fault.GlareFaultWrapperFilter(None)

    def test_no_exception(self):
        req = mock.Mock()
        req.get_response.return_value = 'Response object'
        with mock.patch.object(fault.Fault, '__init__') as mocked_fault:
            res = self._build_middleware()(req)
            self.assertEqual('Response object', res)
            self.assertEqual(0, mocked_fault.call_count)

    def test_exceptions(self):
        req = mock.Mock()
        error_map = fault.GlareFaultWrapperFilter.error_map

        # Raise all exceptions from error_map
        for name, obj in inspect.getmembers(exc, inspect.isclass):
            if not issubclass(obj, Exception)\
                    or obj is exc.InvalidGlobalAPIVersion:
                continue
            req.get_response.side_effect = partial(self.get_response,
                                                   exception=obj)
            res = self._build_middleware()(req)

            while name not in error_map:
                obj = obj.__base__
                name = obj.__name__
            self.assertEqual(error_map[name].code, res.error['code'])

        # Raise other possible exceptions that lead to 500 error
        for e in (Exception, ValueError, TypeError, exc.GlareException):
            req.get_response.side_effect = partial(
                self.get_response, exception=e)
            res = self._build_middleware()(req)
            self.assertEqual(500, res.error['code'])

        # InvalidGlobalAPIVersion should also include min_version and
        # max_version headers
        req.get_response.side_effect = partial(
            self.get_response, exception=exc.InvalidGlobalAPIVersion(
                req_ver=100.0, min_ver=1.0, max_ver=1.1))
        res = self._build_middleware()(req)
        self.assertEqual(406, res.error['code'])
        self.assertEqual(1.0, res.error['min_version'])
        self.assertEqual(1.1, res.error['max_version'])

    def test_trace_marker(self):
        req = mock.Mock()
        self.config(debug=True)
        traceback_marker = 'Traceback (most recent call last)'
        pref = "PREFIX"
        suff = "SUFFIX"

        # Test with marker
        req.get_response.side_effect = partial(
            self.get_response, exception=ValueError(
                pref + traceback_marker + suff))
        res = self._build_middleware()(req)
        self.assertEqual(500, res.error['code'])
        self.assertEqual(pref, res.error['error']['message'])
        self.assertEqual(traceback_marker + suff,
                         res.error['error']['traceback'])

        # Test without marker
        req.get_response.side_effect = partial(
            self.get_response, exception=ValueError(
                pref + suff))
        res = self._build_middleware()(req)
        self.assertEqual(500, res.error['code'])
        self.assertEqual(pref + suff, res.error['error']['message'])
        self.assertIn(traceback_marker, res.error['error']['traceback'])

    def test_fault_class(self):
        req = mock.Mock()
        req.get_response.side_effect = partial(
            self.get_response, exception=exc.BadRequest)
        res = self._build_middleware()(req)(req)
        self.assertEqual(400, res.status_code)
        self.assertEqual('400 Bad Request', res.status)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""A middleware that turns exceptions into parsable string.
Inspired by Cinder's and Heat't faultwrapper.
"""

import sys
import traceback

from oslo_config import cfg
from oslo_log import log as logging
from oslo_middleware import base as base_middleware
from oslo_utils import reflection
import six
import webob.dec
import webob.exc

from glare.common import exception
from glare.common import wsgi


LOG = logging.getLogger(__name__)


class Fault(object):

    def __init__(self, error):
        self.error = error

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        serializer = wsgi.JSONResponseSerializer()
        resp = webob.Response(request=req)
        default_webob_exc = webob.exc.HTTPInternalServerError()
        resp.status_code = self.error.get('code', default_webob_exc.code)
        serializer.default(resp, self.error)
        return resp


class GlareFaultWrapperFilter(base_middleware.ConfigurableMiddleware):
    """Replace error body with something the client can parse."""
    error_map = {
        'BadRequest': webob.exc.HTTPBadRequest,
        'Unauthorized': webob.exc.HTTPUnauthorized,
        'Forbidden': webob.exc.HTTPForbidden,
        'NotFound': webob.exc.HTTPNotFound,
        'RequestTimeout': webob.exc.HTTPRequestTimeout,
        'Conflict': webob.exc.HTTPConflict,
        'Gone': webob.exc.HTTPGone,
        'PreconditionFailed': webob.exc.HTTPPreconditionFailed,
        'RequestEntityTooLarge': webob.exc.HTTPRequestEntityTooLarge,
        'UnsupportedMediaType': webob.exc.HTTPUnsupportedMediaType,
        'RequestRangeNotSatisfiable': webob.exc.HTTPRequestRangeNotSatisfiable,
        'Locked': webob.exc.HTTPLocked,
        'FailedDependency': webob.exc.HTTPFailedDependency,
        'NotAcceptable': webob.exc.HTTPNotAcceptable,
        'Exception': webob.exc.HTTPInternalServerError,
    }

    def _map_exception_to_error(self, class_exception):
        if class_exception.__name__ not in self.error_map:
            return self._map_exception_to_error(class_exception.__base__)

        return self.error_map[class_exception.__name__]

    def _error(self, ex):
        traceback_marker = 'Traceback (most recent call last)'
        webob_exc = None

        ex_type = reflection.get_class_name(ex, fully_qualified=False)

        full_message = six.text_type(ex)
        if traceback_marker in full_message:
            message, msg_trace = full_message.split(traceback_marker, 1)
            message = message.rstrip('\n')
            msg_trace = traceback_marker + msg_trace
        else:
            msg_trace = 'None\n'
            if sys.exc_info() != (None, None, None):
                msg_trace = traceback.format_exc()
            message = full_message

        if isinstance(ex, exception.GlareException):
            message = six.text_type(ex)

        if not webob_exc:
            webob_exc = self._map_exception_to_error(ex.__class__)

        error = {
            'code': webob_exc.code,
            'title': webob_exc.title,
            'explanation': webob_exc.explanation,
            'error': {
                'message': message,
                'type': ex_type,
            }
        }

        if cfg.CONF.debug:
            error['error']['traceback'] = msg_trace

        # add microversion header is this is not acceptable request
        if isinstance(ex, exception.InvalidGlobalAPIVersion):
            error['min_version'] = ex.kwargs['min_ver']
            error['max_version'] = ex.kwargs['max_ver']

        return error

    @webob.dec.wsgify
    def __call__(self, req):
        try:
            return req.get_response(self.application)
        except Exception as exc:
            LOG.exception(exc)
            return req.get_response(Fault(self._error(exc)))



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""WSGI Resource definition for Glare. Defines Glare API and serialization/
deserialization of incoming requests."""

import json
import jsonpatch
import jsonschema
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import encodeutils
import six
from six.moves import http_client
import six.moves.urllib.parse as urlparse

from glare.api.v1 import api_versioning
from glare.common import exception as exc
from glare.common import wsgi
from glare import engine
from glare.i18n import _

LOG = logging.getLogger(__name__)

CONF = cfg.CONF

list_configs = [
    cfg.IntOpt('default_api_limit', default=25,
               help=_('Default value for the number of items returned by a '
                      'request if not specified explicitly in the request')),
    cfg.IntOpt('max_api_limit', default=1000,
               help=_('Maximum permissible number of items that could be '
                      'returned by a request')),
]

CONF.register_opts(list_configs)

supported_versions = api_versioning.VersionedResource.supported_versions

QUOTA_SCHEMA = {
    'type': 'object',
    'properties': {
        'quota_name': {
            u'maxLength': 255,
            u'minLength': 1,
            u'pattern': u'^[^:]*:?[^:]*$',  # can have only 1 or 0 ':'
            u'type': u'string'},
        'quota_value': {'type': 'integer', u'minimum': -1},
    },
    'required': ['quota_name', 'quota_value']
}

QUOTA_INPUT_SCHEMA = {
    "$schema": "http://json-schema.org/draft-04/schema#",
    "items": {
        "properties": {
            "project_id": {
                u'maxLength': 255,
                u'minLength': 1,
                "type": "string"
            },
            "project_quotas": {
                "items": QUOTA_SCHEMA,
                "type": "array"
            }
        },
        "type": "object",
        "required": ["project_id", "project_quotas"]
    },
    "type": "array"
}


class RequestDeserializer(api_versioning.VersionedResource,
                          wsgi.JSONRequestDeserializer):
    """Glare deserializer for incoming webob requests.

    Deserializer checks and converts incoming request into a bunch of Glare
    primitives. So other service components don't work with requests at all.
    Deserializer also performs primary API validation without any knowledge
    about concrete artifact type structure.
    """

    ALLOWED_LOCATION_TYPES = ('external', 'internal')

    @staticmethod
    def _get_content_type(req, expected=None):
        """Determine content type of the request body."""
        if "Content-Type" not in req.headers:
            msg = _("Content-Type must be specified.")
            LOG.error(msg)
            raise exc.BadRequest(msg)

        content_type = req.content_type
        if expected is not None and content_type not in expected:
            msg = (_('Invalid content type: %(ct)s. Expected: %(exp)s') %
                   {'ct': content_type, 'exp': ', '.join(expected)})
            raise exc.UnsupportedMediaType(message=msg)

        return content_type

    @staticmethod
    def _get_content_length(req):
        """Determine content length of the request body."""
        if req.content_length is None:
            return

        try:
            content_length = int(req.content_length)
            if content_length < 0:
                raise ValueError
        except ValueError:
            msg = _("Content-Length must be a non negative integer.")
            LOG.error(msg)
            raise exc.BadRequest(msg)

        return content_length

    def _get_request_body(self, req):
        """Get request json body and convert it to python structures."""
        return self.from_json(req.body)

    @supported_versions(min_ver='1.0')
    def create(self, req):
        self._get_content_type(req, expected=['application/json'])
        body = self._get_request_body(req)
        if not isinstance(body, dict):
            msg = _("Dictionary expected as body value. Got %s.") % type(body)
            raise exc.BadRequest(msg)
        return {'values': body}

    @supported_versions(min_ver='1.0')
    def list(self, req):
        params = req.params.copy()
        marker = params.pop('marker', None)
        query_params = {}
        # step 1 - apply marker to query if exists
        if marker is not None:
            query_params['marker'] = marker

        # step 2 - apply limit (if exists OR setup default limit)
        limit = params.pop('limit', CONF.default_api_limit)
        try:
            limit = int(limit)
        except ValueError:
            msg = _("Limit param must be an integer.")
            raise exc.BadRequest(message=msg)
        if limit < 0:
            msg = _("Limit param must be positive.")
            raise exc.BadRequest(message=msg)
        query_params['limit'] = min(CONF.max_api_limit, limit)

        # step 3 - parse sort parameters
        if 'sort' in params:
            sort = []
            for sort_param in params.pop('sort').strip().split(','):
                key, _sep, direction = sort_param.partition(':')
                if direction and direction not in ('asc', 'desc'):
                    raise exc.BadRequest('Sort direction must be one of '
                                         '["asc", "desc"]. Got %s direction'
                                         % direction)
                sort.append((key, direction or 'desc'))
            query_params['sort'] = sort

        # step 4 - parse filter parameters
        filters = []
        for fname, fval in params.items():
            if fname == 'version' and fval == 'latest':
                query_params['latest'] = True
            else:
                filters.append((fname, fval))

        query_params['filters'] = filters
        return query_params

    @supported_versions(min_ver='1.0')
    def update(self, req):
        self._get_content_type(
            req, expected=['application/json-patch+json'])
        body = self._get_request_body(req)
        patch = jsonpatch.JsonPatch(body)
        try:
            # Initially patch object doesn't validate input. It's only checked
            # when we call get operation on each method
            tuple(map(patch._get_operation, patch.patch))
        except (jsonpatch.InvalidJsonPatch, TypeError, AttributeError,
                jsonpatch.JsonPointerException):
            msg = _("Json Patch body is malformed")
            raise exc.BadRequest(msg)
        return {'patch': patch}

    @supported_versions(min_ver='1.0')
    def upload_blob(self, req):
        content_type = self._get_content_type(req)
        content_length = self._get_content_length(req)
        if content_type == ('application/vnd+openstack.glare-custom-location'
                            '+json'):
            data = self._get_request_body(req)
            if 'url' not in data:
                msg = _("url is required when specifying external location. "
                        "Cannot find 'url' in request body: %s") % str(data)
                raise exc.BadRequest(msg)
            location_type = data.get('location_type', 'external')
            if location_type not in self.ALLOWED_LOCATION_TYPES:
                msg = (_("Incorrect location type '%(location_type)s'. It "
                         "must be one of the following %(allowed)s") %
                       {'location_type': location_type,
                        'allowed': ', '.join(self.ALLOWED_LOCATION_TYPES)})
                raise exc.BadRequest(msg)
            if location_type == 'external':
                url = data.get('url')
                if not url.startswith('http'):
                    msg = _("Url '%s' doesn't have http(s) scheme") % url
                    raise exc.BadRequest(msg)
                if 'md5' not in data:
                    msg = _("Incorrect blob metadata. MD5 must be specified "
                            "for external location in artifact blob.")
                    raise exc.BadRequest(msg)
        else:
            data = req.body_file

        if self.is_valid_encoding(req) and self.is_valid_method(req):
            req.is_body_readable = True

        return {'data': data,
                'content_type': content_type,
                'content_length': content_length}

    @supported_versions(min_ver='1.1')
    def set_quotas(self, req):
        self._get_content_type(req, expected=['application/json'])
        body = self._get_request_body(req)
        try:
            jsonschema.validate(body, QUOTA_INPUT_SCHEMA)
        except jsonschema.exceptions.ValidationError as e:
            raise exc.BadRequest(e)
        values = {}
        for item in body:
            project_id = item['project_id']
            values[project_id] = {}
            for quota in item['project_quotas']:
                values[project_id][quota['quota_name']] = quota['quota_value']
        return {'values': values}

    # TODO(mfedosin) add pagination to list of quotas


def log_request_progress(f):
    def log_decorator(self, req, *args, **kwargs):
        LOG.debug("Request %(request_id)s for %(api_method)s successfully "
                  "deserialized. Pass request parameters to Engine",
                  {'request_id': req.context.request_id,
                   'api_method': f.__name__})
        result = f(self, req, *args, **kwargs)
        LOG.info(
            "Request %(request_id)s for artifact %(api_method)s "
            "successfully executed.", {'request_id': req.context.request_id,
                                       'api_method': f.__name__})
        return result
    return log_decorator


class ArtifactsController(api_versioning.VersionedResource):
    """API controller for Glare Artifacts.

    Artifact Controller prepares incoming data for Glare Engine and redirects
    data to the appropriate engine method. Once the response data is returned
    from the engine Controller passes it next to Response Serializer.
    """

    def __init__(self):
        self.engine = engine.Engine()

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def list_type_schemas(self, req):
        """List of detailed descriptions of enabled artifact types.

        :param req: user request
        :return: list of json-schemas of all enabled artifact types.
        """
        return self.engine.show_type_schemas(req.context)

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def show_type_schema(self, req, type_name):
        """Get detailed artifact type description.

        :param req: user request
        :param type_name: artifact type name
        :return: json-schema representation of artifact type
        """
        type_schema = self.engine.show_type_schemas(req.context, type_name)
        return {type_name: type_schema}

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def create(self, req, type_name, values):
        """Create artifact record in Glare.

        :param req: user request
        :param type_name: artifact type name
        :param values: dict with artifact fields
        :return: definition of created artifact
        """
        if req.context.tenant is None or req.context.read_only:
            msg = _("It's forbidden to anonymous users to create artifacts.")
            raise exc.Forbidden(msg)
        if not values.get('name'):
            msg = _("Name must be specified at creation.")
            raise exc.BadRequest(msg)
        for field in ('visibility', 'status'):
            if field in values:
                msg = _("%s is not allowed in a request at creation.") % field
                raise exc.BadRequest(msg)
        return self.engine.create(req.context, type_name, values)

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def update(self, req, type_name, artifact_id, patch):
        """Update artifact record in Glare.

        :param req: User request
        :param type_name: Artifact type name
        :param artifact_id: id of artifact to update
        :param patch: json patch with artifact changes
        :return: definition of updated artifact
        """
        return self.engine.save(req.context, type_name, artifact_id, patch)

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def delete(self, req, type_name, artifact_id):
        """Delete artifact from Glare.

        :param req: User request
        :param type_name: Artifact type name
        :param artifact_id: id of artifact to delete
        """
        return self.engine.delete(req.context, type_name, artifact_id)

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def show(self, req, type_name, artifact_id):
        """Show detailed artifact info.

        :param req: User request
        :param type_name: Artifact type name
        :param artifact_id: id of artifact to show
        :return: definition of requested artifact
        """
        return self.engine.show(req.context, type_name, artifact_id)

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def list(self, req, type_name, filters=None, marker=None, limit=None,
             sort=None, latest=False):
        """List available artifacts.

        :param req: User request
        :param type_name: Artifact type name
        :param filters: filters that need to be applied to artifact
        :param marker: the artifact that considered as begin of the list
        so all artifacts before marker (including marker itself) will not be
        added to artifact list
        :param limit: maximum number of items in list
        :param sort: sorting options
        :param latest: flag that indicates, that only artifacts with highest
        versions should be returned in output
        :return: list of requested artifact definitions
        """
        artifacts = self.engine.list(req.context, type_name, filters, marker,
                                     limit, sort, latest)
        result = {'artifacts': artifacts,
                  'type_name': type_name}
        if len(artifacts) != 0 and len(artifacts) == limit:
            result['next_marker'] = artifacts[-1]['id']
        return result

    @staticmethod
    def _parse_blob_path(blob_path):
        field_name, _sep, blob_key = blob_path.partition('/')
        if not blob_key:
            blob_key = None
        return field_name, blob_key

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def upload_blob(self, req, type_name, artifact_id, blob_path, data,
                    content_type, content_length=None):
        """Upload blob into Glare repo.

        :param req: User request
        :param type_name: Artifact type name
        :param artifact_id: id of artifact where to perform upload
        :param blob_path: path to artifact blob
        :param data: blob payload
        :param content_type: data content-type
        :param content_length: amount of data user wants to upload
        :return: definition of requested artifact with uploaded blob
        """
        field_name, blob_key = self._parse_blob_path(blob_path)
        if content_type == ('application/vnd+openstack.glare-custom-location'
                            '+json'):
            url = data.pop('url')
            return self.engine.add_blob_location(
                req.context, type_name, artifact_id, field_name, url, data,
                blob_key)
        else:
            return self.engine.upload_blob(
                req.context, type_name, artifact_id, field_name, data,
                content_type, content_length, blob_key)

    @supported_versions(min_ver='1.0')
    @log_request_progress
    def download_blob(self, req, type_name, artifact_id, blob_path):
        """Download blob data from Artifact.

        :param req: User request
        :param type_name: artifact type name
        :param artifact_id: id of artifact from where to perform download
        :param blob_path: path to artifact blob
        :return: requested blob data
        """
        field_name, blob_key = self._parse_blob_path(blob_path)
        data, meta = self.engine.download_blob(
            req.context, type_name, artifact_id, field_name, blob_key)
        result = {'data': data, 'meta': meta}
        return result

    @supported_versions(min_ver='1.1')
    @log_request_progress
    def delete_external_blob(self, req, type_name, artifact_id, blob_path):
        """Delete blob with external location from Glare repo.

        :param req: User request
        :param type_name: Artifact type name
        :param artifact_id: id of artifact with the blob to delete
        :param blob_path: path to artifact blob
        """
        field_name, blob_key = self._parse_blob_path(blob_path)
        return self.engine.delete_external_blob(
            req.context, type_name, artifact_id, field_name, blob_key)

    @supported_versions(min_ver='1.1')
    @log_request_progress
    def set_quotas(self, req, values):
        """Set quota records in Glare.

        :param req: user request
        :param values: list with quota values to set
        """
        self.engine.set_quotas(req.context, values)

    @supported_versions(min_ver='1.1')
    @log_request_progress
    def list_all_quotas(self, req):
        """Get detailed info about all available quotas.

        :param req: user request
        :return: definition of requested quotas for the project
        """
        return self.engine.list_all_quotas(req.context)

    @supported_versions(min_ver='1.1')
    @log_request_progress
    def list_project_quotas(self, req, project_id=None):
        """Get detailed info about project quotas.

        :param req: user request
        :param project_id: id of the project for which to show quotas
        :return: definition of requested quotas for the project
        """
        return self.engine.list_project_quotas(req.context, project_id)


class ResponseSerializer(api_versioning.VersionedResource,
                         wsgi.JSONResponseSerializer):
    """Glare serializer for outgoing responses.

    Converts data received from the engine to WSGI responses. It also
    specifies proper response status and content type as declared in the API.
    """

    @staticmethod
    def _prepare_json_response(response, result,
                               content_type='application/json'):
        body = json.dumps(result, ensure_ascii=False)
        response.text = six.text_type(body)
        response.content_type = content_type + '; charset=UTF-8'

    def list_type_schemas(self, response, type_schemas):
        self._prepare_json_response(response,
                                    {'schemas': type_schemas},
                                    content_type='application/schema+json')

    def show_type_schema(self, response, type_schema):
        self._prepare_json_response(response,
                                    {'schemas': type_schema},
                                    content_type='application/schema+json')

    @supported_versions(min_ver='1.0')
    def list_schemas(self, response, type_list):
        self._prepare_json_response(response, {'types': type_list})

    @supported_versions(min_ver='1.0')
    def create(self, response, artifact):
        self._prepare_json_response(response, artifact)
        response.status_int = http_client.CREATED

    @supported_versions(min_ver='1.0')
    def show(self, response, artifact):
        self._prepare_json_response(response, artifact)

    @supported_versions(min_ver='1.0')
    def update(self, response, artifact):
        self._prepare_json_response(response, artifact)

    @supported_versions(min_ver='1.0')
    def list(self, response, af_list):
        params = dict(response.request.params)
        params.pop('marker', None)

        encode_params = {}
        for key, value in params.items():
            encode_params[key] = encodeutils.safe_encode(value)
        query = urlparse.urlencode(encode_params)

        type_name = af_list['type_name']
        body = {
            type_name: af_list['artifacts'],
            'first': '/artifacts/%s' % type_name,
            'schema': '/schemas/%s' % type_name,
        }
        if query:
            body['first'] = '%s?%s' % (body['first'], query)
        if 'next_marker' in af_list:
            params['marker'] = af_list['next_marker']
            next_query = urlparse.urlencode(params)
            body['next'] = '/artifacts/%s?%s' % (type_name, next_query)

        self._prepare_json_response(response, body)

    @supported_versions(min_ver='1.0')
    def delete(self, response, result):
        response.status_int = http_client.NO_CONTENT

    @supported_versions(min_ver='1.0')
    def upload_blob(self, response, artifact):
        self._prepare_json_response(response, artifact)

    @staticmethod
    def _serialize_blob(response, result):
        data, meta = result['data'], result['meta']
        response.app_iter = iter(data)
        response.headers['Content-Type'] = meta['content_type']
        response.headers['Content-MD5'] = meta['md5']
        response.headers['X-Openstack-Glare-Content-SHA1'] = meta['sha1']
        response.headers['X-Openstack-Glare-Content-SHA256'] = meta['sha256']
        response.content_length = str(meta['size'])

    @staticmethod
    def _serialize_location(response, result):
        data, meta = result['data'], result['meta']
        response.headers['Content-MD5'] = meta['md5']
        response.headers['X-Openstack-Glare-Content-SHA1'] = meta['sha1']
        response.headers['X-Openstack-Glare-Content-SHA256'] = meta['sha256']
        response.location = data['url']
        response.content_type = 'application/json'
        response.status = http_client.MOVED_PERMANENTLY
        response.content_length = 0

    @supported_versions(min_ver='1.0')
    def download_blob(self, response, result):
        external = result['meta']['external']
        if external:
            self._serialize_location(response, result)
        else:
            self._serialize_blob(response, result)

    @supported_versions(min_ver='1.1')
    def delete_external_blob(self, response, result):
        self._prepare_json_response(response, result)

    @staticmethod
    def _serialize_quota(quotas):
        res = []
        for project_id, project_quotas in quotas.items():
            quota_list = []
            for quota_name, quota_value in project_quotas.items():
                quota_list.append({
                    'quota_name': quota_name,
                    'quota_value': quota_value,
                })
            res.append({
                'project_id': project_id,
                'project_quotas': quota_list
            })
        return res

    @supported_versions(min_ver='1.1')
    def list_all_quotas(self, response, quotas):
        quotas['quotas'] = self._serialize_quota(quotas['quotas'])
        self._prepare_json_response(response, quotas)

    @supported_versions(min_ver='1.1')
    def list_project_quotas(self, response, quotas):
        quotas = self._serialize_quota(quotas)
        self._prepare_json_response(response, quotas)


def create_resource():
    """Artifact resource factory method."""
    deserializer = RequestDeserializer()
    serializer = ResponseSerializer()
    controller = ArtifactsController()
    return wsgi.Resource(controller, deserializer, serializer)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from glare.tests.functional import base


class TestQuotasAPI(base.TestArtifact):
    """Test quotas REST API."""

    def setUp(self):
        base.functional.FunctionalTest.setUp(self)

        self.glare_server.deployment_flavor = 'noauth'

        self.glare_server.max_uploaded_data = '10000'
        self.glare_server.max_artifact_number = '150'

        self.glare_server.enabled_artifact_types = 'images,' \
                                                   'heat_templates,' \
                                                   'murano_packages,' \
                                                   'sample_artifact'
        self.glare_server.custom_artifact_types_modules = (
            'glare.tests.sample_artifact')
        self.glare_server.artifact_type_section = """
[artifact_type:sample_artifact]
max_uploaded_data = 3000
[artifact_type:images]
max_uploaded_data = 15000
max_artifact_number = 30
[artifact_type:heat_templates]
max_artifact_number = 150
[artifact_type:murano_packages]
max_uploaded_data = 10000
max_artifact_number = 100
"""
        self.start_servers(**self.__dict__.copy())

    def test_quota_api_wrong(self):
        self.set_user('admin')

        url = '/quotas'
        # try to set wrong values
        values = [{"project1": "value1"}]
        self.put(url=url, data=values, status=400)

        # no quota name
        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_value": 10
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # no quota value
        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # no project id
        values = [
            {
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # no project quotas
        values = [
            {
                "project_id": "project1",
            }
        ]
        self.put(url=url, data=values, status=400)

        # quota name has more than 1 :
        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_name": "max:artifact:number",
                        "quota_value": 10
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # too long quota name
        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_name": "a" * 256,
                        "quota_value": 10
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # too long project name
        values = [
            {
                "project_id": "a" * 256,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # negative quota value less than -1
        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": -2
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        # non-integer quota value
        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": "AAA"
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

        values = [
            {
                "project_id": "project1",
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10.5
                    }
                ]
            }
        ]
        self.put(url=url, data=values, status=400)

    @staticmethod
    def _deserialize_quotas(quotas):
        values = {}
        for item in quotas:
            project_id = item['project_id']
            values[project_id] = {}
            for quota in item['project_quotas']:
                values[project_id][quota['quota_name']] = quota['quota_value']
        return values

    def test_quota_api(self):
        self.set_user('admin')
        user1_tenant_id = self.users['user1']['tenant_id']
        user2_tenant_id = self.users['user2']['tenant_id']
        admin_tenant_id = self.users['admin']['tenant_id']
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number:images",
                        "quota_value": 3
                    },
                    {
                        "quota_name": "max_artifact_number:heat_templates",
                        "quota_value": 15
                    },
                    {
                        "quota_name": "max_artifact_number:murano_packages",
                        "quota_value": 10
                    },
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            },
            {
                "project_id": user2_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            },
            {
                "project_id": admin_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            }
        ]

        url = '/quotas'
        # define several quotas
        self.put(url=url, data=values)

        # get all quotas
        res = self.get(url=url)
        global_quotas = res['global_quotas']
        self.assertEqual({
            'max_artifact_number': 150,
            'max_artifact_number:heat_templates': 150,
            'max_artifact_number:images': 30,
            'max_artifact_number:murano_packages': 100,
            'max_uploaded_data': 10000,
            'max_uploaded_data:images': 15000,
            'max_uploaded_data:murano_packages': 10000,
            'max_uploaded_data:sample_artifact': 3000}, global_quotas)
        self.assertEqual(self._deserialize_quotas(values),
                         self._deserialize_quotas(res['quotas']))

        # get user1 quotas
        res = self._deserialize_quotas(self.get(
            url='/project-quotas/' + user1_tenant_id))
        self.assertEqual({user1_tenant_id: {
            'max_artifact_number': 10,
            'max_artifact_number:heat_templates': 15,
            'max_artifact_number:images': 3,
            'max_artifact_number:murano_packages': 10,
            'max_uploaded_data': 10000,
            'max_uploaded_data:images': 15000,
            'max_uploaded_data:murano_packages': 10000,
            'max_uploaded_data:sample_artifact': 3000}}, res)

        # get admin quotas
        res = self._deserialize_quotas(self.get(url='/project-quotas'))
        self.assertEqual({admin_tenant_id: {
            'max_artifact_number': 10,
            'max_artifact_number:heat_templates': 150,
            'max_artifact_number:images': 30,
            'max_artifact_number:murano_packages': 100,
            'max_uploaded_data': 10000,
            'max_uploaded_data:images': 15000,
            'max_uploaded_data:murano_packages': 10000,
            'max_uploaded_data:sample_artifact': 3000}}, res)

        # user1 can't set quotas
        self.set_user('user1')
        self.put(url=url, data=values, status=403)
        self.get(url=url, status=403)

        # user1 can get his quotas
        res = self._deserialize_quotas(self.get(url='/project-quotas'))
        self.assertEqual({user1_tenant_id: {
            'max_artifact_number': 10,
            'max_artifact_number:heat_templates': 15,
            'max_artifact_number:images': 3,
            'max_artifact_number:murano_packages': 10,
            'max_uploaded_data': 10000,
            'max_uploaded_data:images': 15000,
            'max_uploaded_data:murano_packages': 10000,
            'max_uploaded_data:sample_artifact': 3000}}, res)

        # user1 can't get user2 quotas
        self.get(url='/project-quotas/' + user2_tenant_id, status=403)


class TestStaticQuotas(base.TestArtifact):
    """Test static quota limits."""

    def setUp(self):
        base.functional.FunctionalTest.setUp(self)

        self.set_user('user1')
        self.glare_server.deployment_flavor = 'noauth'
        self.glare_server.max_uploaded_data = '1000'
        self.glare_server.max_artifact_number = '10'

        self.glare_server.enabled_artifact_types = 'images,' \
                                                   'heat_templates,' \
                                                   'murano_packages,' \
                                                   'sample_artifact'
        self.glare_server.custom_artifact_types_modules = (
            'glare.tests.sample_artifact')
        self.glare_server.artifact_type_section = """
[artifact_type:sample_artifact]
default_store = database
max_uploaded_data = 300
[artifact_type:images]
max_uploaded_data = 1500
max_artifact_number = 3
[artifact_type:heat_templates]
max_artifact_number = 15
[artifact_type:murano_packages]
max_uploaded_data = 1000
max_artifact_number = 10
"""
        self.start_servers(**self.__dict__.copy())

    def test_count_artifact_number(self):
        # initially there are no artifacts
        result = self.get('/all')
        self.assertEqual([], result['all'])

        # create 3 images for user1
        for i in range(3):
            img = self.create_artifact(
                data={'name': 'img%d' % i}, type_name='images')

        # creation of another image fails because of artifact type limit
        self.create_artifact(
            data={'name': 'img4'}, type_name='images', status=403)

        # create 7 murano packages
        for i in range(7):
            self.create_artifact(
                data={'name': 'mp%d' % i}, type_name='murano_packages')

        # creation of another package fails because of global limit
        self.create_artifact(
            data={'name': 'mp8'}, type_name='murano_packages', status=403)

        # delete an image and create another murano package work
        self.delete('/images/%s' % img['id'])
        self.create_artifact(
            data={'name': 'mp8'}, type_name='murano_packages')

        # admin can create his own artifacts
        self.set_user('admin')
        for i in range(10):
            self.create_artifact(
                data={'name': 'ht%d' % i}, type_name='heat_templates')

        # creation of another heat template fails because of global limit
        self.create_artifact(
            data={'name': 'ht11'}, type_name='heat_templates', status=403)

    def test_calculate_uploaded_data(self):
        headers = {'Content-Type': 'application/octet-stream'}

        # initially there are no artifacts
        result = self.get('/all')
        self.assertEqual([], result['all'])

        # create 2 sample artifacts for user1
        art1 = self.create_artifact(data={'name': 'art1'})
        art2 = self.create_artifact(data={'name': 'art2'})

        # create 2 images for user1
        img1 = self.create_artifact(data={'name': 'img1'}, type_name='images')
        img2 = self.create_artifact(data={'name': 'img2'}, type_name='images')

        # upload to art1 fails now because of type limit
        data = 'a' * 301
        self.put(url='/sample_artifact/%s/blob' % art1['id'],
                 data=data,
                 status=413,
                 headers=headers)

        # upload to img1 fails now because of global limit
        data = 'a' * 1001
        self.put(url='/images/%s/image' % img1['id'],
                 data=data,
                 status=413,
                 headers=headers)

        # upload 300 bytes to 'blob' of art1
        data = 'a' * 300
        self.put(url='/sample_artifact/%s/blob' % art1['id'],
                 data=data,
                 headers=headers)

        # upload another blob to art1 fails because of type limit
        self.put(url='/sample_artifact/%s/dict_of_blobs/blob' % art1['id'],
                 data='a',
                 status=413,
                 headers=headers)

        # upload to art2 fails now because of type limit
        self.put(url='/sample_artifact/%s/dict_of_blobs/blob' % art2['id'],
                 data='a',
                 status=413,
                 headers=headers)

        # delete art1 and check that upload to art2 works
        data = 'a' * 300
        self.delete('/sample_artifact/%s' % art1['id'])
        self.put(url='/sample_artifact/%s/dict_of_blobs/blob' % art2['id'],
                 data=data,
                 headers=headers)

        # upload 700 bytes to img1 works
        data = 'a' * 700
        self.put(url='/images/%s/image' % img1['id'],
                 data=data,
                 headers=headers)

        # upload to img2 fails because of global limit
        self.put(url='/images/%s/image' % img2['id'],
                 data='a',
                 status=413,
                 headers=headers)

        # admin can upload data to his images
        self.set_user('admin')
        img1 = self.create_artifact(data={'name': 'img1'}, type_name='images')
        data = 'a' * 1000
        self.put(url='/images/%s/image' % img1['id'],
                 data=data,
                 headers=headers)


class TestDynamicQuotas(base.TestArtifact):
    """Test dynamic quota limits."""

    def setUp(self):
        base.functional.FunctionalTest.setUp(self)

        self.glare_server.deployment_flavor = 'noauth'

        self.glare_server.enabled_artifact_types = 'images,' \
                                                   'heat_templates,' \
                                                   'murano_packages,' \
                                                   'sample_artifact'
        self.glare_server.custom_artifact_types_modules = (
            'glare.tests.sample_artifact')
        self.start_servers(**self.__dict__.copy())

    def test_count_artifact_number(self):
        self.set_user('admin')
        user1_tenant_id = self.users['user1']['tenant_id']
        admin_tenant_id = self.users['admin']['tenant_id']
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number:images",
                        "quota_value": 3
                    },
                    {
                        "quota_name": "max_artifact_number:heat_templates",
                        "quota_value": 15
                    },
                    {
                        "quota_name": "max_artifact_number:murano_packages",
                        "quota_value": 10
                    },
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            },
            {
                "project_id": admin_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": 10
                    }
                ]
            }
        ]
        url = '/quotas'
        # define several quotas
        self.put(url=url, data=values)

        self.set_user('user1')
        # initially there are no artifacts
        result = self.get('/all')
        self.assertEqual([], result['all'])

        # create 3 images for user1
        for i in range(3):
            img = self.create_artifact(
                data={'name': 'img%d' % i}, type_name='images')

        # creation of another image fails because of artifact type limit
        self.create_artifact(
            data={'name': 'img4'}, type_name='images', status=403)

        # create 7 murano packages
        for i in range(7):
            self.create_artifact(
                data={'name': 'mp%d' % i}, type_name='murano_packages')

        # creation of another package fails because of global limit
        self.create_artifact(
            data={'name': 'mp8'}, type_name='murano_packages', status=403)

        # delete an image and create another murano package work
        self.delete('/images/%s' % img['id'])
        self.create_artifact(
            data={'name': 'mp8'}, type_name='murano_packages')

        # admin can create his own artifacts
        self.set_user('admin')
        for i in range(10):
            self.create_artifact(
                data={'name': 'ht%d' % i}, type_name='heat_templates')

        # creation of another heat template fails because of global limit
        self.create_artifact(
            data={'name': 'ht11'}, type_name='heat_templates', status=403)

        # disable global limit for user1 and try to create 15 heat templates
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number:images",
                        "quota_value": 3
                    },
                    {
                        "quota_name": "max_artifact_number:heat_templates",
                        "quota_value": 15
                    },
                    {
                        "quota_name": "max_artifact_number:murano_packages",
                        "quota_value": 10
                    },
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": -1
                    }
                ]
            }
        ]
        url = '/quotas'
        self.put(url=url, data=values)

        self.set_user("user1")
        for i in range(15):
            self.create_artifact(
                data={'name': 'ht%d' % i}, type_name='heat_templates')

        # creation of another heat template fails because of type limit
        self.create_artifact(
            data={'name': 'ht16'}, type_name='heat_templates', status=403)

        self.set_user("admin")
        # disable type limit for heat templates and create 1 heat templates
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_artifact_number:images",
                        "quota_value": 3
                    },
                    {
                        "quota_name": "max_artifact_number:heat_templates",
                        "quota_value": -1
                    },
                    {
                        "quota_name": "max_artifact_number:murano_packages",
                        "quota_value": 10
                    },
                    {
                        "quota_name": "max_artifact_number",
                        "quota_value": -1
                    }
                ]
            }
        ]
        url = '/quotas'
        self.put(url=url, data=values)

        # now user1 can create another heat template
        self.set_user("user1")
        self.create_artifact(
            data={'name': 'ht16'}, type_name='heat_templates')

    def test_calculate_uploaded_data(self):
        self.set_user('admin')
        user1_tenant_id = self.users['user1']['tenant_id']
        admin_tenant_id = self.users['admin']['tenant_id']
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_uploaded_data:images",
                        "quota_value": 1500
                    },
                    {
                        "quota_name": "max_uploaded_data:sample_artifact",
                        "quota_value": 300
                    },
                    {
                        "quota_name": "max_uploaded_data:murano_packages",
                        "quota_value": 1000
                    },
                    {
                        "quota_name": "max_uploaded_data",
                        "quota_value": 1000
                    }
                ]
            },
            {
                "project_id": admin_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_uploaded_data",
                        "quota_value": 1000
                    }
                ]
            }
        ]
        url = '/quotas'
        # define several quotas
        self.put(url=url, data=values)

        headers = {'Content-Type': 'application/octet-stream'}

        self.set_user('user1')
        # initially there are no artifacts
        result = self.get('/all')
        self.assertEqual([], result['all'])

        # create 2 sample artifacts for user1
        art1 = self.create_artifact(data={'name': 'art1'})
        art2 = self.create_artifact(data={'name': 'art2'})

        # create 3 images for user1
        img1 = self.create_artifact(data={'name': 'img1'}, type_name='images')
        img2 = self.create_artifact(data={'name': 'img2'}, type_name='images')
        img3 = self.create_artifact(data={'name': 'img3'}, type_name='images')

        # upload to art1 fails now because of type limit
        data = 'a' * 301
        self.put(url='/sample_artifact/%s/blob' % art1['id'],
                 data=data,
                 status=413,
                 headers=headers)

        # upload to img1 fails now because of global limit
        data = 'a' * 1001
        self.put(url='/images/%s/image' % img1['id'],
                 data=data,
                 status=413,
                 headers=headers)

        # upload 300 bytes to 'blob' of art1
        data = 'a' * 300
        self.put(url='/sample_artifact/%s/blob' % art1['id'],
                 data=data,
                 headers=headers)

        # upload another blob to art1 fails because of type limit
        self.put(url='/sample_artifact/%s/dict_of_blobs/blob' % art1['id'],
                 data='a',
                 status=413,
                 headers=headers)

        # upload to art2 fails now because of type limit
        self.put(url='/sample_artifact/%s/dict_of_blobs/blob' % art2['id'],
                 data='a',
                 status=413,
                 headers=headers)

        # delete art1 and check that upload to art2 works
        data = 'a' * 300
        self.delete('/sample_artifact/%s' % art1['id'])
        self.put(url='/sample_artifact/%s/dict_of_blobs/blob' % art2['id'],
                 data=data,
                 headers=headers)

        # upload 700 bytes to img1 works
        data = 'a' * 700
        self.put(url='/images/%s/image' % img1['id'],
                 data=data,
                 headers=headers)

        # upload to img2 fails because of global limit
        self.put(url='/images/%s/image' % img2['id'],
                 data='a',
                 status=413,
                 headers=headers)

        # admin can upload data to his images
        self.set_user('admin')
        img1 = self.create_artifact(data={'name': 'img1'}, type_name='images')
        data = 'a' * 1000
        self.put(url='/images/%s/image' % img1['id'],
                 data=data,
                 headers=headers)

        # disable global limit and try upload data from user1 again
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_uploaded_data:images",
                        "quota_value": 1500
                    },
                    {
                        "quota_name": "max_uploaded_data:sample_artifact",
                        "quota_value": 300
                    },
                    {
                        "quota_name": "max_uploaded_data:murano_packages",
                        "quota_value": 1000
                    },
                    {
                        "quota_name": "max_uploaded_data",
                        "quota_value": -1
                    }
                ]
            }
        ]
        url = '/quotas'
        self.put(url=url, data=values)

        self.set_user("user1")
        data = 'a' * 800
        self.put(url='/images/%s/image' % img2['id'],
                 data=data,
                 headers=headers)

        # uploading more fails because of image type limit
        data = 'a'
        self.put(url='/images/%s/image' % img3['id'],
                 data=data,
                 headers=headers,
                 status=413)

        # disable type limit and try upload data from user1 again
        self.set_user("admin")
        values = [
            {
                "project_id": user1_tenant_id,
                "project_quotas": [
                    {
                        "quota_name": "max_uploaded_data:images",
                        "quota_value": -1
                    },
                    {
                        "quota_name": "max_uploaded_data:sample_artifact",
                        "quota_value": 300
                    },
                    {
                        "quota_name": "max_uploaded_data:murano_packages",
                        "quota_value": 1000
                    },
                    {
                        "quota_name": "max_uploaded_data",
                        "quota_value": -1
                    }
                ]
            }
        ]
        url = '/quotas'
        self.put(url=url, data=values)

        self.set_user("user1")
        data = 'a' * 1000
        self.put(url='/images/%s/image' % img3['id'],
                 data=data,
                 headers=headers)



#!/usr/bin/env python

# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""
Glare Scrub Service
"""

import os
import sys

# If ../glare/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'glare', '__init__.py')):
    sys.path.insert(0, possible_topdir)
import eventlet

import glance_store
from oslo_config import cfg
from oslo_log import log as logging

from glare.common import config
from glare import scrubber

eventlet.patcher.monkey_patch(all=False, socket=True, time=True, select=True,
                              thread=True, os=True)

CONF = cfg.CONF
logging.register_options(CONF)
CONF.set_default(name='use_stderr', default=True)


def main():
    CONF.register_cli_opts(scrubber.scrubber_cmd_cli_opts, group='scrubber')
    CONF.register_opts(scrubber.scrubber_cmd_opts, group='scrubber')

    try:
        config.parse_args()
        logging.setup(CONF, 'glare')

        glance_store.register_opts(config.CONF)
        glance_store.create_stores(config.CONF)
        glance_store.verify_default_store()

        app = scrubber.Scrubber()

        if CONF.scrubber.daemon:
            server = scrubber.Daemon(CONF.scrubber.wakeup_time)
            server.start(app)
            server.wait()
        else:
            app.run()
    except RuntimeError as e:
        sys.exit("ERROR: %s" % e)


if __name__ == '__main__':
    main()



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Glare WSGI module.

Use this module to deploy glare as WSGI application.

Sample usage with uwsgi:

    export GLARE_CONFIG_FILE=/etc/glare/glare.conf
    uwsgi --module glare.wsgi:application --socket 127.0.0.1:8008

Sample apache mod_wsgi configuration:

    <VirtualHost *:80>
         ServerName example.com
         SetEnv GLARE_CONFIG_FILE=/etc/glare/glare.conf
         DocumentRoot /path/to/public_html/
         WSGIScriptAlias / /usr/lib/python2.7/site-packages/glare/wsgi.py
         ...
    </VirtualHost>

"""

import os

from oslo_config import cfg
from oslo_log import log as logging

from glare.common import config
from glare.common import utils


CONF = cfg.CONF
logging.register_options(CONF)
CONFIG_FILE = os.environ.get("GLARE_CONFIG_FILE", "etc/glare.conf")
config.parse_args(args=["--config-file", CONFIG_FILE])

utils.initialize_glance_store()

application = config.load_paste_app('glare-api')



# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base test class for running non-stubbed tests (functional tests)

The FunctionalTest class contains helper methods for starting Glare
server, grabbing the logs of each, cleaning up pidfiles, and spinning down
the server.
"""

import atexit
import datetime
import errno
import os
import platform
import shutil
import signal
import socket
import sys
import tempfile
import time

import eventlet
import fixtures
from oslo_log import log as logging
from oslo_serialization import jsonutils
# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
from six.moves import range
import six.moves.urllib.parse as urlparse
import testtools

from glare.api.v1 import resource
from glare.api.v1 import router
from glare.common import utils
from glare.common import wsgi
from glare.db.sqlalchemy import api as db_api
from glare import tests as glare_tests
from glare.tests import utils as test_utils

execute, get_unused_port = test_utils.execute, test_utils.get_unused_port
tracecmd_osmap = {'Linux': 'strace', 'FreeBSD': 'truss'}

eventlet.patcher.monkey_patch()


class Server(object):
    """Class used to easily manage starting and stopping
    a server during functional test runs.
    """
    def __init__(self, test_dir, port, sock=None):
        """Creates a new Server object.

        :param test_dir: The directory where all test stuff is kept. This is
                         passed from the FunctionalTestCase.
        :param port: The port to start a server up on.
        """
        self.debug = True
        self.no_venv = False
        self.test_dir = test_dir
        self.bind_port = port
        self.conf_file_name = None
        self.conf_base = None
        self.paste_conf_base = None
        self.exec_env = None
        self.deployment_flavor = ''
        self.needs_database = False
        self.log_file = None
        self.sock = sock
        self.fork_socket = True
        self.process_pid = None
        self.server_module = None
        self.stop_kill = False

    def write_conf(self, **kwargs):
        """Writes the configuration file for the server to its intended
        destination.  Returns the name of the configuration file and
        the over-ridden config content (may be useful for populating
        error messages).
        """
        if not self.conf_base:
            raise RuntimeError("Subclass did not populate config_base!")

        conf_override = self.__dict__.copy()
        if kwargs:
            conf_override.update(**kwargs)

        # A config file and paste.ini to use just for this test...we don't want
        # to trample on currently-running Glare servers, now do we?

        conf_dir = os.path.join(self.test_dir, 'etc')
        conf_filepath = os.path.join(conf_dir, "%s.conf" % self.server_name)
        if os.path.exists(conf_filepath):
            os.unlink(conf_filepath)
        paste_conf_filepath = conf_filepath.replace(".conf", "-paste.ini")
        if os.path.exists(paste_conf_filepath):
            os.unlink(paste_conf_filepath)
        test_utils.safe_mkdirs(conf_dir)

        def override_conf(filepath, overridden):
            with open(filepath, 'w') as conf_file:
                conf_file.write(overridden)
                conf_file.flush()
                return conf_file.name

        overridden_core = self.conf_base % conf_override
        self.conf_file_name = override_conf(conf_filepath, overridden_core)

        overridden_paste = ''
        if self.paste_conf_base:
            overridden_paste = self.paste_conf_base % conf_override
            override_conf(paste_conf_filepath, overridden_paste)

        overridden = ('==Core config==\n%s\n==Paste config==\n%s' %
                      (overridden_core, overridden_paste))

        return self.conf_file_name, overridden

    def start(self, expect_exit=True, expected_exitcode=0, **kwargs):
        """Starts the server.

        Any kwargs passed to this method will override the configuration
        value in the conf file used in starting the servers.
        """

        # Ensure the configuration file is written
        self.write_conf(**kwargs)

        self.create_database()

        cmd = ("%(server_module)s --config-file %(conf_file_name)s"
               % {"server_module": self.server_module,
                  "conf_file_name": self.conf_file_name})
        cmd = "%s -m %s" % (sys.executable, cmd)
        # close the sock and release the unused port closer to start time
        if self.exec_env:
            exec_env = self.exec_env.copy()
        else:
            exec_env = {}
        pass_fds = set()
        if self.sock:
            if not self.fork_socket:
                self.sock.close()
                self.sock = None
            else:
                fd = os.dup(self.sock.fileno())
                exec_env[utils.GLARE_TEST_SOCKET_FD_STR] = str(fd)
                pass_fds.add(fd)
                self.sock.close()

        self.process_pid = test_utils.fork_exec(cmd,
                                                logfile=os.devnull,
                                                exec_env=exec_env,
                                                pass_fds=pass_fds)

        self.stop_kill = not expect_exit
        if self.pid_file:
            with open(self.pid_file, 'w') as pf:
                pf.write('%d\n' % self.process_pid)
        if not expect_exit:
            rc = 0
            try:
                os.kill(self.process_pid, 0)
            except OSError:
                raise RuntimeError("The process did not start")
        else:
            rc = test_utils.wait_for_fork(
                self.process_pid,
                expected_exitcode=expected_exitcode)
        # avoid an FD leak
        if self.sock:
            os.close(fd)
            self.sock = None
        return (rc, '', '')

    def reload(self, expect_exit=True, expected_exitcode=0, **kwargs):
        """Start and stop the service to reload

        Any kwargs passed to this method will override the configuration
        value in the conf file used in starting the servers.
        """
        self.stop()
        return self.start(expect_exit=expect_exit,
                          expected_exitcode=expected_exitcode, **kwargs)

    def create_database(self):
        """Create database if required for this server"""
        if self.needs_database:
            conf_dir = os.path.join(self.test_dir, 'etc')
            test_utils.safe_mkdirs(conf_dir)
            conf_filepath = os.path.join(conf_dir, 'glare.conf')

            glare_db_env = 'GLARE_DB_TEST_SQLITE_FILE'
            if glare_db_env in os.environ:
                # use the empty db created and cached as a tempfile
                # instead of spending the time creating a new one
                db_location = os.environ[glare_db_env]
                os.system('cp %s %s/tests.sqlite'
                          % (db_location, self.test_dir))
            else:
                cmd = ('%s -m glare.cmd.db_manage --config-file %s upgrade' %
                       (sys.executable, conf_filepath))
                execute(cmd, no_venv=self.no_venv, exec_env=self.exec_env,
                        expect_exit=True)

                # copy the clean db to a temp location so that it
                # can be reused for future tests
                (osf, db_location) = tempfile.mkstemp()
                os.close(osf)
                os.system('cp %s/tests.sqlite %s'
                          % (self.test_dir, db_location))
                os.environ[glare_db_env] = db_location

                # cleanup the temp file when the test suite is
                # complete
                def _delete_cached_db():
                    try:
                        os.remove(os.environ[glare_db_env])
                    except Exception:
                        glare_tests.logger.exception(
                            "Error cleaning up the file %s" %
                            os.environ[glare_db_env])
                atexit.register(_delete_cached_db)

    def stop(self):
        """Spin down the server."""
        if not self.process_pid:
            raise Exception('why is this being called? %s' % self.server_name)

        if self.stop_kill:
            os.kill(self.process_pid, signal.SIGTERM)
        rc = test_utils.wait_for_fork(self.process_pid, raise_error=False)
        return (rc, '', '')

    def dump_log(self, name):
        log = logging.getLogger(name)
        if not self.log_file or not os.path.exists(self.log_file):
            return
        with open(self.log_file, 'r') as fptr:
            for line in fptr:
                log.info(line.strip())


class GlareServer(Server):

    """Server object that starts/stops/manages Glare server"""

    def __init__(self, test_dir, port, policy_file, delayed_delete=False,
                 pid_file=None, sock=None, **kwargs):
        super(GlareServer, self).__init__(test_dir, port, sock=sock)
        self.server_name = 'glare'
        self.server_module = 'glare.cmd.api'
        self.default_store = kwargs.get("default_store", "file")
        self.key_file = ""
        self.cert_file = ""
        self.blob_dir = os.path.join(self.test_dir, "artifacts")
        self.pid_file = pid_file or os.path.join(self.test_dir, "glare.pid")
        self.log_file = os.path.join(self.test_dir, "glare.log")
        self.delayed_delete = delayed_delete
        self.workers = 1
        self.policy_file = policy_file
        self.policy_default_rule = 'default'
        self.disable_path = None

        self.needs_database = True
        default_sql_connection = 'sqlite:////%s/tests.sqlite' % self.test_dir
        self.sql_connection = os.environ.get('GLARE_TEST_SQL_CONNECTION',
                                             default_sql_connection)
        self.lock_path = self.test_dir

        self.enabled_artifact_types = ''
        self.custom_artifact_types_modules = ''
        self.max_uploaded_data = '1099511627776'
        self.max_artifact_number = '100'
        self.artifact_type_section = ''

        self.conf_base = """[DEFAULT]
debug = %(debug)s
default_log_levels = eventlet.wsgi.server=DEBUG
bind_host = 127.0.0.1
bind_port = %(bind_port)s
key_file = %(key_file)s
cert_file = %(cert_file)s
log_file = %(log_file)s
delayed_delete = %(delayed_delete)s
workers = %(workers)s
lock_path = %(lock_path)s
enabled_artifact_types = %(enabled_artifact_types)s
custom_artifact_types_modules = %(custom_artifact_types_modules)s
max_uploaded_data = %(max_uploaded_data)s
max_artifact_number = %(max_artifact_number)s
[oslo_policy]
policy_file = %(policy_file)s
policy_default_rule = %(policy_default_rule)s
[paste_deploy]
flavor = %(deployment_flavor)s
[glance_store]
filesystem_store_datadir=%(blob_dir)s
default_store = %(default_store)s
[database]
connection = %(sql_connection)s
%(artifact_type_section)s
"""
        self.paste_conf_base = """[pipeline:glare-api]
pipeline = faultwrapper versionnegotiation trustedauth glarev1api

[pipeline:glare-api-noauth]
pipeline = faultwrapper versionnegotiation context glarev1api

[app:glarev1api]
paste.app_factory =
 glare.tests.functional:TestRouter.factory

[filter:faultwrapper]
paste.filter_factory =
 glare.api.middleware.fault:GlareFaultWrapperFilter.factory

[filter:versionnegotiation]
paste.filter_factory =
 glare.api.middleware.version_negotiation:
   GlareVersionNegotiationFilter.factory

[filter:context]
paste.filter_factory = glare.api.middleware.context:ContextMiddleware.factory

[filter:trustedauth]
paste.filter_factory =
 glare.api.middleware.context:TrustedAuthMiddleware.factory
"""


class ScrubberDaemon(Server):
    """
    Server object that starts/stops/manages the Scrubber server
    """

    def __init__(self, test_dir, policy_file, daemon=False, **kwargs):
        # NOTE(jkoelker): Set the port to 0 since we actually don't listen
        super(ScrubberDaemon, self).__init__(test_dir, 0)
        self.server_name = 'scrubber'
        self.server_module = 'glare.cmd.%s' % self.server_name
        self.daemon = daemon

        self.blob_dir = os.path.join(self.test_dir, "artifacts")
        self.scrub_time = 5
        self.pid_file = os.path.join(self.test_dir, "scrubber.pid")
        self.log_file = os.path.join(self.test_dir, "scrubber.log")
        self.lock_path = self.test_dir

        default_sql_connection = 'sqlite:////%s/tests.sqlite' % self.test_dir
        self.sql_connection = os.environ.get('GLARE_TEST_SQL_CONNECTION',
                                             default_sql_connection)
        self.policy_file = policy_file
        self.policy_default_rule = 'default'

        self.conf_base = """[DEFAULT]
debug = %(debug)s
log_file = %(log_file)s
[scrubber]
daemon = %(daemon)s
wakeup_time = 2
scrub_time = %(scrub_time)s
[glance_store]
filesystem_store_datadir=%(blob_dir)s
[oslo_policy]
policy_file = %(policy_file)s
policy_default_rule = %(policy_default_rule)s
[database]
connection = %(sql_connection)s
idle_timeout = 3600
"""

    def start(self, expect_exit=True, expected_exitcode=0, **kwargs):
        if 'daemon' in kwargs:
            expect_exit = False
        return super(ScrubberDaemon, self).start(
            expect_exit=expect_exit,
            expected_exitcode=expected_exitcode,
            **kwargs)


class FunctionalTest(test_utils.BaseTestCase):

    """Base test class for any test that wants to test the actual
    servers and clients and not just the stubbed out interfaces
    """

    inited = False
    disabled = False
    launched_servers = []

    def setUp(self):
        super(FunctionalTest, self).setUp()
        self.test_dir = self.useFixture(fixtures.TempDir()).path

        self.api_protocol = 'http'
        self.glare_port, glare_sock = test_utils.get_unused_port_and_socket()

        self.include_scrubber = False

        self.tracecmd = tracecmd_osmap.get(platform.system())

        conf_dir = os.path.join(self.test_dir, 'etc')
        test_utils.safe_mkdirs(conf_dir)
        self.copy_data_file('policy.json', conf_dir)
        self.policy_file = os.path.join(conf_dir, 'policy.json')

        self.glare_server = GlareServer(self.test_dir,
                                        self.glare_port,
                                        self.policy_file,
                                        sock=glare_sock)

        self.scrubber_daemon = ScrubberDaemon(self.test_dir, self.policy_file)

        self.pid_files = [self.glare_server.pid_file,
                          self.scrubber_daemon.pid_file]
        self.files_to_destroy = []
        self.launched_servers = []

    def tearDown(self):
        if not self.disabled:
            self.cleanup()
            # We destroy the test data store between each test case,
            # and recreate it, which ensures that we have no side-effects
            # from the tests
            self._reset_database(self.glare_server.sql_connection)
        super(FunctionalTest, self).tearDown()

        self.glare_server.dump_log('glare_server')
        self.scrubber_daemon.dump_log('scrubber_daemon')

    def set_policy_rules(self, rules):
        with open(self.policy_file, 'w') as fap:
            fap.write(jsonutils.dumps(rules))

    def _reset_database(self, conn_string):
        conn_pieces = urlparse.urlparse(conn_string)
        if conn_string.startswith('sqlite'):
            # We leave behind the sqlite DB for failing tests to aid
            # in diagnosis, as the file size is relatively small and
            # won't interfere with subsequent tests as it's in a per-
            # test directory (which is blown-away if the test is green)
            pass
        elif conn_string.startswith('mysql'):
            # We can execute the MySQL client to destroy and re-create
            # the MYSQL database, which is easier and less error-prone
            # than using SQLAlchemy to do this via MetaData...trust me.
            database = conn_pieces.path.strip('/')
            loc_pieces = conn_pieces.netloc.split('@')
            host = loc_pieces[1]
            auth_pieces = loc_pieces[0].split(':')
            user = auth_pieces[0]
            password = ""
            if len(auth_pieces) > 1:
                if auth_pieces[1].strip():
                    password = "-p%s" % auth_pieces[1]
            sql = ("drop database if exists %(database)s; "
                   "create database %(database)s;") % {'database': database}
            cmd = ("mysql -u%(user)s %(password)s -h%(host)s "
                   "-e\"%(sql)s\"") % {'user': user, 'password': password,
                                       'host': host, 'sql': sql}
            exitcode, out, err = execute(cmd)
            self.assertEqual(0, exitcode)

    def cleanup(self):
        """Makes sure anything we created or started up in the
        tests are destroyed or spun down
        """

        # NOTE(jbresnah) call stop on each of the servers instead of
        # checking the pid file.  stop() will wait until the child
        # server is dead.  This eliminates the possibility of a race
        # between a child process listening on a port actually dying
        # and a new process being started
        servers = [self.glare_server,
                   self.scrubber_daemon]
        for s in servers:
            try:
                s.stop()
            except Exception:
                pass

        for f in self.files_to_destroy:
            if os.path.exists(f):
                os.unlink(f)

    def start_server(self,
                     server,
                     expect_launch,
                     expect_exit=True,
                     expected_exitcode=0,
                     **kwargs):
        """Starts a server on an unused port.

        Any kwargs passed to this method will override the configuration
        value in the conf file used in starting the server.

        :param server: the server to launch
        :param expect_launch: true iff the server is expected to
                              successfully start
        :param expect_exit: true iff the launched process is expected
                            to exit in a timely fashion
        :param expected_exitcode: expected exitcode from the launcher
        """
        self.cleanup()

        # Start up the requested server
        exitcode, out, err = server.start(expect_exit=expect_exit,
                                          expected_exitcode=expected_exitcode,
                                          **kwargs)
        if expect_exit:
            self.assertEqual(expected_exitcode, exitcode,
                             "Failed to spin up the requested server. "
                             "Got: %s" % err)

        self.launched_servers.append(server)

        launch_msg = self.wait_for_servers([server], expect_launch)
        self.assertTrue(launch_msg is None, launch_msg)

    def start_with_retry(self, server, port_name, max_retries,
                         expect_launch=True,
                         **kwargs):
        """Starts a server, with retries if the server launches but
        fails to start listening on the expected port.

        :param server: the server to launch
        :param port_name: the name of the port attribute
        :param max_retries: the maximum number of attempts
        :param expect_launch: true iff the server is expected to
                              successfully start
        :param expect_exit: true iff the launched process is expected
                            to exit in a timely fashion
        """
        launch_msg = None
        for i in range(max_retries):
            exitcode, out, err = server.start(expect_exit=not expect_launch,
                                              **kwargs)
            name = server.server_name
            self.assertEqual(0, exitcode,
                             "Failed to spin up the %s server. "
                             "Got: %s" % (name, err))
            launch_msg = self.wait_for_servers([server], expect_launch)
            if launch_msg:
                server.stop()
                server.bind_port = get_unused_port()
                setattr(self, port_name, server.bind_port)
            else:
                self.launched_servers.append(server)
                break
        self.assertTrue(launch_msg is None, launch_msg)

    def start_servers(self, **kwargs):
        """Starts the Glare server on unused port.

        Any kwargs passed to this method will override the configuration
        value in the conf file used in starting the servers.
        """
        self.cleanup()

        self.start_with_retry(self.glare_server, 'glare_port', 3, **kwargs)

        if self.include_scrubber:
            exitcode, out, err = self.scrubber_daemon.start(**kwargs)
            self.assertEqual(0, exitcode,
                             "Failed to spin up the Scrubber daemon. "
                             "Got: %s" % err)

    def ping_server(self, port):
        """Simple ping on the port. If responsive, return True, else
        return False.

        :note We use raw sockets, not ping here, since ping uses ICMP and
        has no concept of ports...
        """
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect(("127.0.0.1", port))
            return True
        except socket.error:
            return False
        finally:
            s.close()

    def wait_for_servers(self, servers, expect_launch=True, timeout=30):
        """Tight loop, waiting for the given server port(s) to be available.
        Returns when all are pingable. There is a timeout on waiting
        for the servers to come up.

        :param servers: Glare server ports to ping
        :param expect_launch: Optional, true iff the server(s) are
                              expected to successfully start
        :param timeout: Optional, defaults to 30 seconds
        :returns: None if launch expectation is met, otherwise an
                 assertion message
        """
        now = datetime.datetime.now()
        timeout_time = now + datetime.timedelta(seconds=timeout)
        replied = []
        while (timeout_time > now):
            pinged = 0
            for server in servers:
                if self.ping_server(server.bind_port):
                    pinged += 1
                    if server not in replied:
                        replied.append(server)
            if pinged == len(servers):
                msg = 'Unexpected server launch status'
                return None if expect_launch else msg
            now = datetime.datetime.now()
            time.sleep(0.05)

        failed = list(set(servers) - set(replied))
        msg = 'Unexpected server launch status for: '
        for f in failed:
            msg += ('%s, ' % f.server_name)
            if os.path.exists(f.pid_file):
                pid = f.process_pid
                trace = f.pid_file.replace('.pid', '.trace')
                if self.tracecmd:
                    cmd = '%s -p %d -o %s' % (self.tracecmd, pid, trace)
                    try:
                        execute(cmd, raise_error=False, expect_exit=False)
                    except OSError as e:
                        if e.errno == errno.ENOENT:
                            raise RuntimeError('No executable found for "%s" '
                                               'command.' % self.tracecmd)
                        else:
                            raise
                    time.sleep(0.5)
                    if os.path.exists(trace):
                        msg += ('\n%s:\n%s\n' % (self.tracecmd,
                                                 open(trace).read()))

        self.add_log_details(failed)

        return msg if expect_launch else None

    def stop_server(self, server):
        """Called to stop a single server in a normal fashion.

        :param server: the server to stop
        """
        # Spin down the requested server
        server.stop()

    def stop_servers(self):
        self.stop_server(self.glare_server)

        if self.include_scrubber:
            self.stop_server(self.scrubber_daemon)

        self._reset_database(self.glare_server.sql_connection)

    def run_sql_cmd(self, sql):
        """Provides a crude mechanism to run manual SQL commands
        for backend DB verification within the functional tests.
        The raw result set is returned.
        """
        engine = db_api.get_engine()
        return engine.execute(sql)

    def copy_data_file(self, file_name, dst_dir):
        src_file_name = os.path.join('glare/tests/etc', file_name)
        shutil.copy(src_file_name, dst_dir)
        dst_file_name = os.path.join(dst_dir, file_name)
        return dst_file_name

    def add_log_details(self, servers=None):
        logs = [s.log_file for s in (servers or self.launched_servers)]
        for log in logs:
            if os.path.exists(log):
                testtools.content.attach_file(self, log)


class TestRouter(router.API):
    def _get_artifacts_resource(self):
        deserializer = resource.RequestDeserializer()
        serializer = resource.ResponseSerializer()
        controller = resource.ArtifactsController()
        return wsgi.Resource(controller, deserializer, serializer)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from glance_store import backend
from glance_store import exceptions as store_exc
from oslo_config import cfg
from oslo_log import log as logging

from glare.common import exception
from glare.common import utils
from glare.store import database

CONF = cfg.CONF
LOG = logging.getLogger(__name__)
database_api = database.DatabaseStoreAPI()
# we restrict several schemes because of potential security bugs.
# read more about the bugs here:
# https://bugs.launchpad.net/glance/+bug/942118
# https://bugs.launchpad.net/glance/+bug/1400966
# https://bugs.launchpad.net/glance/+bug/1334196
RESTRICTED_URI_SCHEMES = ('file', 'filesystem', 'swift+config', 'sql')

error_map = [{'catch': store_exc.NotFound,
              'raise': exception.NotFound},
             {'catch': store_exc.UnknownScheme,
              'raise': exception.BadRequest},
             {'catch': store_exc.BadStoreUri,
              'raise': exception.BadRequest},
             {'catch': store_exc.Duplicate,
              'raise': exception.Conflict},
             {'catch': store_exc.StorageFull,
              'raise': exception.Forbidden},
             {'catch': store_exc.StorageWriteDenied,
              'raise': exception.Forbidden},
             {'catch': store_exc.Forbidden,
              'raise': exception.Forbidden},
             {'catch': store_exc.Invalid,
              'raise': exception.BadRequest},
             {'catch': store_exc.BadStoreConfiguration,
              'raise': exception.GlareException},
             {'catch': store_exc.RemoteServiceUnavailable,
              'raise': exception.BadRequest},
             {'catch': store_exc.HasSnapshot,
              'raise': exception.Conflict},
             {'catch': store_exc.InUseByStore,
              'raise': exception.Conflict},
             {'catch': store_exc.BackendException,
              'raise': exception.GlareException},
             {'catch': store_exc.GlanceStoreException,
              'raise': exception.GlareException}]


@utils.error_handler(error_map)
def save_blob_to_store(blob_id, blob, context, max_size,
                       store_type=None, verifier=None):
    """Save file to specified store type and return location info to the user.

    :param store_type: type of the store, None means save to default store.
    :param blob_id: id of blob
    :param blob: blob file iterator
    :param context: user context
    :param verifier:signature verified
    :return: tuple of values: (location_uri, size, checksums)
    """
    data = utils.LimitingReader(utils.CooperativeReader(blob), max_size)

    LOG.debug('Start uploading blob %s.', blob_id)
    if store_type == 'database':
        location = database_api.add_to_backend(
            blob_id, data, context, verifier)
    else:
        (location, size, md5checksum, __) = backend.add_to_backend(
            CONF, blob_id, data, 0, store_type, context, verifier)
    LOG.debug('Uploading of blob %s is finished.', blob_id)

    checksums = {"md5": data.md5.hexdigest(),
                 "sha1": data.sha1.hexdigest(),
                 "sha256": data.sha256.hexdigest()}
    return location, data.bytes_read, checksums


@utils.error_handler(error_map)
def save_blobs_to_store(blobs, context, max_size,
                        store_type=None, verifier=None):
    """Save several files to specified store.

    :param store_type: type of the store, None means save to default store.
    :param blobs: list of tuples (blob_data_id, data)
    :param context: user context
    :param verifier:signature verified
    :return: dict {blob_data_id: (location_uri, size, checksums)}
    """
    # wrap data in CooperativeReader
    blobs = [(blob_data_id,
              utils.LimitingReader(utils.CooperativeReader(data), max_size))
             for (blob_data_id, data) in blobs]

    if store_type == 'database':
        locations = database_api.add_to_backend_batch(blobs, context, verifier)
    else:
        locations = []
        for blob_data_id, data in blobs:
            (location, __, __, __) = backend.add_to_backend(
                CONF, blob_data_id, data, 0, store_type, context, verifier)
            locations.append(location)

    # combine location, size and checksums together
    res = {}
    for i in range(len(locations)):
        data = blobs[i][1]
        checksums = {"md5": data.md5.hexdigest(),
                     "sha1": data.sha1.hexdigest(),
                     "sha256": data.sha256.hexdigest()}
        res[blobs[i][0]] = (locations[i], data.bytes_read, checksums)

    return res


@utils.error_handler(error_map)
def load_from_store(uri, context):
    """Load file from store backend.

    :param uri: blob uri
    :param context: user context
    :return: file iterator
    """
    if uri.startswith("sql://"):
        return utils.BlobIterator(
            database_api.get_from_store(uri, context))
    return backend.get_from_backend(uri=uri, context=context)[0]


@utils.error_handler(error_map)
def delete_blob(uri, context):
    """Delete blob from backend store.

    :param uri: blob uri
    :param context: user context
    """
    if uri.startswith("sql://"):
        return database_api.delete_from_store(uri, context)
    return backend.delete_from_backend(uri, context)


def get_known_schemes():
    return list(backend.get_known_schemes()) + ['sql']


def read_data(flobj, limit=16777216):
    """Read data into memory from the file-like object.

    :param flobj: file-like object that contains data
    :param limit: max file size that can be read into memory
    :return: string with data from the object
    """
    bytes_read = 0
    data = b''
    for chunk in flobj:
        bytes_read += len(chunk)
        if bytes_read > limit:
            raise exception.RequestEntityTooLarge()
        data += chunk
    return data



# Copyright 2011-2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Glare policy operations inspired by Nova implementation."""

from oslo_config import cfg
from oslo_log import log as logging
from oslo_policy import policy

from glare.common import exception

CONF = cfg.CONF
LOG = logging.getLogger(__name__)

_ENFORCER = None


artifact_policy_rules = [
    policy.RuleDefault('context_is_admin', 'role:admin'),
    policy.RuleDefault('admin_or_owner',
                       'is_admin:True or project_id:%(owner)s'),
    policy.RuleDefault("artifact:type_list", "",
                       "Policy to request list of artifact types"),
    policy.RuleDefault("artifact:create", "", "Policy to create artifact."),
    policy.RuleDefault("artifact:update_public",
                       "'public':%(visibility)s and rule:context_is_admin "
                       "or not 'public':%(visibility)s",
                       "Policy to update public artifact"),
    policy.RuleDefault("artifact:update", "rule:admin_or_owner and "
                                          "rule:artifact:update_public",
                       "Policy to update artifact"),
    policy.RuleDefault("artifact:activate", "rule:admin_or_owner",
                       "Policy to activate artifact"),
    policy.RuleDefault("artifact:reactivate", "rule:context_is_admin",
                       "Policy to reactivate artifact"),
    policy.RuleDefault("artifact:deactivate", "rule:context_is_admin",
                       "Policy to update artifact"),
    policy.RuleDefault("artifact:publish", "rule:context_is_admin",
                       "Policy to publish artifact"),
    policy.RuleDefault("artifact:get", "",
                       "Policy to get artifact definition"),
    policy.RuleDefault("artifact:list", "",
                       "Policy to list artifacts"),
    policy.RuleDefault("artifact:delete_public",
                       "'public':%(visibility)s and rule:context_is_admin "
                       "or not 'public':%(visibility)s",
                       "Policy to delete public artifacts"),
    policy.RuleDefault("artifact:delete_deactivated",
                       "'deactivated':%(status)s and rule:context_is_admin "
                       "or not 'deactivated':%(status)s",
                       "Policy to delete deactivated artifacts"),
    policy.RuleDefault("artifact:delete", "rule:admin_or_owner and "
                                          "rule:artifact:delete_public and "
                                          "rule:artifact:delete_deactivated",
                       "Policy to delete artifacts"),
    policy.RuleDefault("artifact:set_location", "rule:admin_or_owner",
                       "Policy to set custom location for artifact blob"),
    policy.RuleDefault("artifact:set_internal_location",
                       "rule:context_is_admin",
                       "Policy to set internal location for artifact blob"),
    policy.RuleDefault("artifact:upload", "rule:admin_or_owner",
                       "Policy to upload blob for artifact"),
    policy.RuleDefault("artifact:download_deactivated",
                       "'deactivated':%(status)s and rule:context_is_admin "
                       "or not 'deactivated':%(status)s",
                       "Policy to download blob from deactivated artifact"),
    policy.RuleDefault("artifact:download",
                       "rule:admin_or_owner and "
                       "rule:artifact:download_deactivated",
                       "Policy to download blob from artifact"),
    policy.RuleDefault("artifact:delete_blob", "rule:admin_or_owner",
                       "Policy to delete blob with external location "
                       "from artifact"),
    policy.RuleDefault("artifact:set_quotas", "rule:context_is_admin",
                       "Policy to set quotas for projects"),
    policy.RuleDefault("artifact:list_all_quotas", "rule:context_is_admin",
                       "Policy to list all quotas for all projects"),
    policy.RuleDefault("artifact:list_project_quotas",
                       "project_id:%(project_id)s or rule:context_is_admin",
                       "Policy to get info about project quotas"),
]


def list_rules():
    return artifact_policy_rules


def init(use_conf=True):
    """Init an Enforcer class.
    """

    global _ENFORCER
    if not _ENFORCER:
        _ENFORCER = policy.Enforcer(CONF, use_conf=use_conf)
        _ENFORCER.register_defaults(list_rules())
    return _ENFORCER


def reset():
    global _ENFORCER
    if _ENFORCER:
        _ENFORCER.clear()
        _ENFORCER = None


def authorize(policy_name, target, context, do_raise=True):
    """Method checks that user action can be executed according to policies.

    :param policy_name: policy name
    :param target:
    :param do_raise
    :param context:
    :return: True if check passed
    """
    creds = context.to_policy_values()
    result = init().authorize(
        policy_name, target, creds, do_raise=do_raise,
        exc=exception.PolicyException, policy_name=policy_name)
    LOG.debug("Policy %(policy)s check %(result)s for request %(request_id)s",
              {'policy': policy_name,
               'result': 'passed' if result else 'failed',
               'request_id': context.request_id})
    return result


def check_is_admin(context):
    """Whether or not roles contains 'admin' role according to policy setting.
    """
    return authorize('context_is_admin', {}, context, do_raise=False)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from glance_store import exceptions as store_exc
import mock
from six import BytesIO

from glare.common import exception as exc
from glare.common import store_api
from glare.db import artifact_api
from glare.tests import sample_artifact
from glare.tests.unit import base


class TestArtifactUpload(base.BaseTestArtifactAPI):
    """Test blob uploading."""

    def setUp(self):
        super(TestArtifactUpload, self).setUp()
        values = {'name': 'ttt', 'version': '1.0'}
        self.sample_artifact = self.controller.create(
            self.req, 'sample_artifact', values)

    def test_upload_basic(self):
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['blob']['size'])
        self.assertEqual('active', artifact['blob']['status'])

    def test_blob_size_too_big(self):
        # small blob size is limited by 10 bytes
        self.assertRaises(
            exc.RequestEntityTooLarge, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'small_blob', BytesIO(b'a' * 11), 'application/octet-stream')

    def test_already_uploaded(self):
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['blob']['size'])
        self.assertEqual('active', artifact['blob']['status'])

        # Re-uploading blob leads to Conflict error
        self.assertRaises(
            exc.Conflict, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')

    def test_upload_saving_blob(self):
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')
        self.sample_artifact = self.controller.show(
            self.req, 'sample_artifact', self.sample_artifact['id'])

        # Change status of the blob to 'saving'
        self.sample_artifact['blob']['status'] = 'saving'
        artifact_api.ArtifactAPI().update_blob(
            self.req.context, self.sample_artifact['id'],
            {'blob': self.sample_artifact['blob']})
        self.sample_artifact = self.controller.show(
            self.req, 'sample_artifact', self.sample_artifact['id'])
        self.assertEqual('saving', self.sample_artifact['blob']['status'])

        # Uploading new blob leads to Conflict error
        self.assertRaises(
            exc.Conflict, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')

    def test_storage_error(self):
        self.config(default_store='filesystem',
                    group='artifact_type:sample_artifact')
        with mock.patch('glance_store.backend.add_to_backend',
                        side_effect=store_exc.GlanceStoreException):
            self.assertRaises(
                exc.GlareException, self.controller.upload_blob,
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'blob', BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertIsNone(artifact['blob'])

    def test_upload_blob_dict(self):
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blb1',
            BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['dict_of_blobs']['blb1']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['blb1']['status'])

        # upload another one
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blb2',
            BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['dict_of_blobs']['blb2']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['blb2']['status'])

    def test_upload_oversized_blob_dict(self):
        # dict_of_blobs has a limit in 2000 bytes in it

        # external location shouldn't affect folder size
        ct = 'application/vnd+openstack.glare-custom-location+json'
        body = {'url': 'https://FAKE_LOCATION.com',
                'md5': "fake", 'sha1': "fake_sha", "sha256": "fake_sha256"}
        artifact = self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/external', body, ct)
        self.assertIsNone(artifact['dict_of_blobs']['external']['size'])
        self.assertEqual('active',
                         artifact['dict_of_blobs']['external']['status'])

        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/a',
            BytesIO(1800 * b'a'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(1800, artifact['dict_of_blobs']['a']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['a']['status'])

        # upload another one
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/b',
            BytesIO(199 * b'b'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(199, artifact['dict_of_blobs']['b']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['b']['status'])

        # upload to have size of 2000 bytes exactly
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/c',
            BytesIO(b'c'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(1, artifact['dict_of_blobs']['c']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['c']['status'])

        # Upload to have more than max folder limit, more than 2000
        self.assertRaises(
            exc.RequestEntityTooLarge, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/d', BytesIO(b'd'), 'application/octet-stream')

    def test_upload_with_content_length(self):
        # dict_of_blobs has a limit in 2000 bytes in it

        # external location shouldn't affect folder size
        ct = 'application/vnd+openstack.glare-custom-location+json'
        body = {'url': 'https://FAKE_LOCATION.com',
                'md5': "fake", 'sha1': "fake_sha", "sha256": "fake_sha256"}
        artifact = self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/external', body, ct)
        self.assertIsNone(artifact['dict_of_blobs']['external']['size'])
        self.assertEqual('active',
                         artifact['dict_of_blobs']['external']['status'])

        # Error if we provide a content length bigger than max folder size
        with mock.patch('glare.common.store_api.save_blob_to_store') as m:
            self.assertRaises(
                exc.RequestEntityTooLarge, self.controller.upload_blob,
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'dict_of_blobs/d', BytesIO(b'd' * 2001),
                'application/octet-stream', content_length=2001)
            # Check that upload hasn't started
            self.assertEqual(0, m.call_count)

        # Try to cheat and provide content length lesser than we want to upload
        with mock.patch('glare.common.store_api.save_blob_to_store',
                        side_effect=store_api.save_blob_to_store) as m:
            self.assertRaises(
                exc.RequestEntityTooLarge, self.controller.upload_blob,
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'dict_of_blobs/d', BytesIO(b'd' * 2001),
                'application/octet-stream', content_length=100)
            # Check that upload was called this time
            self.assertEqual(1, m.call_count)

        # Upload lesser amount of data works
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/a',
            BytesIO(b'a' * 1800), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(1800, artifact['dict_of_blobs']['a']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['a']['status'])

        # Now we have only 200 bytes left
        # Uploading of 201 byte fails immediately
        with mock.patch('glare.common.store_api.save_blob_to_store') as m:
            self.assertRaises(
                exc.RequestEntityTooLarge, self.controller.upload_blob,
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'dict_of_blobs/d', BytesIO(b'd' * 201),
                'application/octet-stream', content_length=201)
            # Check that upload hasn't started
            self.assertEqual(0, m.call_count)

    def test_existing_blob_dict_key(self):
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blb', BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['dict_of_blobs']['blb']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['blb']['status'])

        # If blob key already exists Glare return Conflict error
        self.assertRaises(
            exc.Conflict, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blb', BytesIO(b'aaa'), 'application/octet-stream')

    def test_blob_dict_storage_error(self):
        self.config(default_store='filesystem',
                    group='artifact_type:sample_artifact')
        with mock.patch('glance_store.backend.add_to_backend',
                        side_effect=store_exc.GlanceStoreException):
            self.assertRaises(
                exc.GlareException, self.controller.upload_blob,
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'dict_of_blobs/blb', BytesIO(b'aaa'),
                'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertNotIn('blb', artifact['dict_of_blobs'])

    def test_upload_with_hook(self):
        with mock.patch.object(
                sample_artifact.SampleArtifact, 'pre_upload_hook',
                return_value=BytesIO(b'ffff')):
            self.controller.upload_blob(
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'blob', BytesIO(b'aaa'), 'application/octet-stream')
            artifact = self.controller.show(self.req, 'sample_artifact',
                                            self.sample_artifact['id'])
            self.assertEqual(4, artifact['blob']['size'])
            self.assertEqual('active', artifact['blob']['status'])

    def test_upload_with_hook_error(self):
        with mock.patch.object(
                sample_artifact.SampleArtifact, 'pre_upload_hook',
                side_effect=Exception):
            self.assertRaises(
                exc.BadRequest, self.controller.upload_blob,
                self.req, 'sample_artifact', self.sample_artifact['id'],
                'dict_of_blobs/blb', BytesIO(b'aaa'),
                'application/octet-stream')
            art = self.controller.show(self.req, 'sample_artifact',
                                       self.sample_artifact['id'])
            self.assertEqual({}, art['dict_of_blobs'])

    def test_upload_nonexistent_field(self):
        self.assertRaises(
            exc.BadRequest, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'], 'INVALID',
            BytesIO(b'aaa'), 'application/octet-stream')

        self.assertRaises(
            exc.BadRequest, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'blob/key', BytesIO(b'aaa'), 'application/octet-stream')

    def test_upload_non_blob_field(self):
        self.assertRaises(
            exc.BadRequest, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'], 'int1',
            BytesIO(b'aaa'), 'application/octet-stream')

    def test_upload_blob_dict_without_key(self):
        self.assertRaises(
            exc.BadRequest, self.controller.upload_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/', BytesIO(b'aaa'), 'application/octet-stream')

    def test_parallel_uploading_and_activation(self):
        """
        This test check whether it is possible to activate an artifact,
        when it has uploading blobs.
        """
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')
        self.sample_artifact = self.controller.show(
            self.req, 'sample_artifact', self.sample_artifact['id'])
        changes = [{'op': 'replace',
                    'path': '/string_required',
                    'value': 'ttt'}]
        self.update_with_values(changes)

        # Change status of the blob to 'saving'
        self.sample_artifact['blob']['status'] = 'saving'
        artifact_api.ArtifactAPI().update_blob(
            self.req.context, self.sample_artifact['id'],
            {'blob': self.sample_artifact['blob']})
        self.sample_artifact = self.controller.show(
            self.req, 'sample_artifact', self.sample_artifact['id'])
        self.assertEqual('saving', self.sample_artifact['blob']['status'])

        # activation of artifact with saving blobs lead to Conflict error
        changes = [{'op': 'replace', 'path': '/status', 'value': 'active'}]
        self.assertRaises(exc.Conflict, self.update_with_values, changes)

        # create another artifact which doesn't have uploading blobs
        values = {'name': 'ttt', 'version': '2.0', 'string_required': 'rrr'}
        new_artifact = self.controller.create(
            self.req, 'sample_artifact', values)
        # activation is possible
        res = self.update_with_values(changes, art_id=new_artifact['id'])
        self.assertEqual('active', res['status'])



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import log as logging
import six

from glare.i18n import _

LOG = logging.getLogger(__name__)


class GlareException(Exception):
    """Base Glare Exception class.

    To correctly use this class, inherit from it and define
    a 'message' property. That message will get printf'd
    with the keyword arguments provided to the constructor.
    """
    message = _("An unknown exception occurred")

    def __init__(self, message=None, **kwargs):
        if message:
            self.message = message
        self.kwargs = kwargs
        if self.kwargs:
            self.message = self.message % kwargs
        LOG.error(self.message)
        super(GlareException, self).__init__(self.message)

    def __unicode__(self):
        return six.text_type(self.message)


class BadRequest(GlareException):
    message = _("Bad request")


class InvalidParameterValue(BadRequest):
    message = _("Invalid filter value ")


class InvalidFilterOperatorValue(BadRequest):
    msg = _("Unable to filter by unknown operator.")


class InvalidVersion(GlareException):
    message = _("Provided version is invalid")


class NotAcceptable(GlareException):
    message = _("Not acceptable")


class InvalidGlobalAPIVersion(NotAcceptable):
    message = _("Version %(req_ver)s is not supported by the API. Minimum "
                "is %(min_ver)s and maximum is %(max_ver)s.")


class VersionNotFoundForAPIMethod(GlareException):
    message = _("API version %(version)s is not supported on this method.")


class ApiVersionsIntersect(GlareException):
    message = _("Version of %(name)s %(min_ver)s %(max_ver)s intersects "
                "with another versions.")


class Unauthorized(GlareException):
    message = _('You are not authenticated')


class Forbidden(GlareException):
    message = _("You are not authorized to complete this action.")


class PolicyException(Forbidden):
    message = _("Policy check for %(policy_name)s "
                "failed with user credentials.")


class NotFound(GlareException):
    message = _("An object with the specified identifier was not found.")


class TypeNotFound(NotFound):
    message = _("Glare type with name '%(name)s' was not found.")


class IncorrectArtifactType(GlareException):
    message = _("Artifact type is incorrect: %(explanation)s")


class ArtifactNotFound(NotFound):
    message = _("Artifact with type name '%(type_name)s' and id '%(id)s' was "
                "not found.")


class RequestTimeout(GlareException):
    message = _("The client did not produce a request within the time "
                "that the server was prepared to wait.")


class Conflict(GlareException):
    message = _("The request could not be completed due to a conflict "
                "with the current state of the resource.")


class Gone(GlareException):
    message = _("The requested resource is no longer available at the "
                "server and no forwarding address is known.")


class PreconditionFailed(GlareException):
    message = _("The precondition given in one or more of the request-header "
                "fields evaluated to false when it was tested on the server.")


class RequestEntityTooLarge(GlareException):
    message = _("The server is refusing to process a request because the "
                "request entity is larger than the server is willing or "
                "able to process.")


class RequestRangeNotSatisfiable(GlareException):
    message = _("The request included a Range request-header field, and none "
                "of the range-specifier values in this field overlap the "
                "current extent of the selected resource, and the request "
                "did not include an If-Range request-header field.")


class Locked(GlareException):
    message = _('The resource is locked.')


class FailedDependency(GlareException):
    message = _('The method could not be performed because the requested '
                'action depended on another action and that action failed.')


class UnsupportedMediaType(GlareException):
    message = _("Unsupported media type.")


class SIGHUPInterrupt(GlareException):
    message = _("System SIGHUP signal received.")


class WorkerCreationFailure(GlareException):
    message = _("Server worker creation failed: %(reason)s.")


class DBNotAllowed(GlareException):
    msg_fmt = _('This operation is not allowed with current DB')



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from glare.objects import base
from glare.objects.meta import wrappers

Blob = wrappers.BlobField.init


class HeatEnvironment(base.BaseArtifact):

    fields = {
        'environment': Blob(description="Heat Environment text body."),
    }

    @classmethod
    def get_type_name(cls):
        return "heat_environments"



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import webob

from glare.api.middleware import version_negotiation
from glare.common import exception as exc
from glare.tests.unit import base


class TestContextMiddleware(base.BaseTestCase):
    MIME_TYPE = 'application/vnd.openstack.artifacts-'

    def _build_request(self, accept, path_info):
        req = webob.Request.blank(path_info)
        req.accept = accept
        return req

    def _build_middleware(self):
        return version_negotiation.GlareVersionNegotiationFilter(None)

    def test_version_request(self):
        _LINKS = [{
            "rel": "describedby",
            "type": "text/html",
            "href": "http://docs.openstack.org/",
        }]
        for path_info in ('/', '/versions'):
            expected = {'versions': [
                {
                    'version': '1.0',
                    'status': 'STABLE',
                    'links': _LINKS,
                    'media-type': 'application/vnd.openstack.artifacts-1.0',
                },
                {
                    'version': '1.1',
                    'status': 'EXPERIMENTAL',
                    'links': _LINKS,
                    'media-type': 'application/vnd.openstack.artifacts-1.1',
                }]
            }
            req = self._build_request(self.MIME_TYPE + '1.0', path_info)
            res = self._build_middleware().process_request(req)
            self.assertEqual(expected, res.json_body)

    def test_wrong_version(self):
        req = self._build_request(self.MIME_TYPE + 'INVALID', '/artifacts')
        self.assertRaises(exc.BadRequest,
                          self._build_middleware().process_request, req)

    def test_too_big_version(self):
        req = self._build_request(self.MIME_TYPE + '10000.0', '/artifacts')
        self.assertRaises(exc.InvalidGlobalAPIVersion,
                          self._build_middleware().process_request, req)

    def test_latest_version(self):
        req = self._build_request(self.MIME_TYPE + 'latest', '/artifacts')
        self._build_middleware().process_request(req)
        self.assertEqual('1.1', req.api_version_request.get_string())

    def test_version_unknown(self):
        req = self._build_request('UNKNOWN', '/artifacts')
        self._build_middleware().process_request(req)
        self.assertEqual('1.0', req.api_version_request.get_string())

    def test_response(self):
        res = webob.Response()
        req = self._build_request('1.0', '/artifacts')
        mw = self._build_middleware()
        mw.process_request(req)
        mw.process_response(res, req)
        self.assertIn('openstack-api-version', res.headers)
        self.assertEqual('artifact 1.0', res.headers['openstack-api-version'])
        self.assertIn('Vary', res.headers)
        self.assertEqual('openstack-api-version', res.headers['Vary'])



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_versionedobjects import base
from oslo_versionedobjects import fields

from glare.common import exception
from glare.common import utils
from glare.db import artifact_api
from glare.i18n import _
from glare.objects.meta import fields as glare_fields
from glare.objects.meta import validators
from glare.objects.meta import wrappers

global_artifact_opts = [
    cfg.IntOpt('max_uploaded_data', default=1099511627776,  # 1 Terabyte
               min=-1,
               help=_("Defines how many bytes of data user can upload to "
                      "storage. This parameter is global and doesn't take "
                      "into account data of what type was uploaded. "
                      "Value -1 means no limit.")),
    cfg.IntOpt('max_artifact_number', default=100,
               min=-1,
               help=_("Defines how many artifacts user can have. This "
                      "parameter is global and doesn't take "
                      "into account artifacts of what type were created. "
                      "Value -1 means no limit.")),
    cfg.BoolOpt('delayed_delete', default=False,
                help=_("If False defines that artifacts must be deleted "
                       "immediately after the user call. Otherwise they just "
                       "will be marked as deleted so they can be scrubbed "
                       "by some other tool in the background.")),
]

CONF = cfg.CONF
CONF.register_opts(global_artifact_opts)

LOG = logging.getLogger(__name__)


class BaseArtifact(base.VersionedObject):
    """BaseArtifact is a central place in Glare. It execute Glare business
    logic operations and checks in like:
    1) Check if artifact satisfies all requirements and can be activated
    2) Check that artifact is not deactivated and download blobs
    ...
    BaseArtifact interacts with database and saves/request artifact info
    from specified database API. Base Artifact is an abstract class so
    all concrete classes must be inherited from that class. Concrete classes
    must define custom fields in addition to BaseArtifact fields and db_api
    that must be used for interaction with database.
    """

    OBJ_PROJECT_NAMESPACE = 'glare'

    DEFAULT_ARTIFACT_VERSION = '0.0.0'

    STATUS = ('drafted', 'active', 'deactivated', 'deleted')

    Field = wrappers.Field.init
    DictField = wrappers.DictField.init
    ListField = wrappers.ListField.init
    Blob = wrappers.BlobField.init

    fields = {
        'id': Field(fields.StringField, system=True,
                    validators=[validators.UUID()], nullable=False,
                    sortable=True, description="Artifact UUID."),
        'name': Field(fields.StringField, required_on_activate=False,
                      nullable=False, sortable=True,
                      validators=[validators.MinStrLen(1)],
                      description="Artifact Name."),
        'owner': Field(fields.StringField, system=True,
                       required_on_activate=False, nullable=False,
                       sortable=True, description="ID of user/tenant who "
                                                  "uploaded artifact."),
        'status': Field(fields.StringField, default='drafted',
                        nullable=False, sortable=True, mutable=True,
                        validators=[validators.AllowedValues(STATUS)],
                        description="Artifact status."),
        'created_at': Field(fields.DateTimeField, system=True,
                            nullable=False, sortable=True,
                            description="Datetime when artifact has "
                                        "been created."),
        'updated_at': Field(fields.DateTimeField, system=True,
                            nullable=False, sortable=True, mutable=True,
                            description="Datetime when artifact has "
                                        "been updated last time."),
        'activated_at': Field(fields.DateTimeField, system=True,
                              required_on_activate=False, sortable=True,
                              description="Datetime when artifact has became "
                                          "active."),
        'description': Field(fields.StringField, mutable=True,
                             required_on_activate=False, default="",
                             validators=[validators.MaxStrLen(4096)],
                             filter_ops=[],
                             description="Artifact description."),
        'tags': ListField(fields.String, mutable=True,
                          required_on_activate=False,
                          # tags are filtered without any operators
                          filter_ops=[],
                          validators=[validators.Unique(convert_to_set=True)],
                          element_validators=[
                              validators.ForbiddenChars([',', '/']),
                              validators.MinStrLen(1)
                          ],
                          description="List of tags added to Artifact."),
        'metadata': DictField(fields.String, required_on_activate=False,
                              element_validators=[validators.MinStrLen(1)],
                              description="Key-value dict with useful "
                                          "information about an artifact."),
        'visibility': Field(fields.StringField, default='private',
                            nullable=False, sortable=True, mutable=True,
                            validators=[validators.AllowedValues(
                                ['private', 'public'])],
                            description="Artifact visibility that defines "
                                        "if artifact can be available to "
                                        "other users."),
        'version': Field(glare_fields.VersionField, required_on_activate=False,
                         default=DEFAULT_ARTIFACT_VERSION, nullable=False,
                         sortable=True, validators=[validators.Version()],
                         description="Artifact version(semver).")
    }

    common_artifact_type_opts = [
        cfg.IntOpt('max_uploaded_data', min=-1,
                   help=_("Defines how many bytes of data of this type user "
                          "can upload to storage. Value -1 means no limit.")),
        cfg.IntOpt('max_artifact_number', min=-1,
                   help=_("Defines how many artifacts of this type user can "
                          "have. Value -1 means no limit.")),
        cfg.BoolOpt('delayed_delete',
                    help=_(
                        "If False defines that artifacts must be deleted "
                        "immediately after the user call. Otherwise they just "
                        "will be marked as deleted so they can be scrubbed "
                        "by some other tool in the background. "
                        "Redefines global parameter of the same name "
                        "from [DEFAULT] section.")),
        cfg.StrOpt('default_store',
                   choices=('file', 'filesystem', 'http', 'https', 'swift',
                            'swift+http', 'swift+https', 'swift+config', 'rbd',
                            'sheepdog', 'cinder', 'vsphere', 'database'),
                   help=_("""
The default scheme to use for storing artifacts of this
type.
Provide a string value representing the default scheme to
use for storing artifact data. If not set, Glare uses
default_store parameter from [glance_store] section.
NOTE: The value given for this configuration option must
be a valid scheme for a store registered with the ``stores``
configuration option.
Possible values:
   * file
   * filesystem
   * http
   * https
   * swift
   * swift+http
   * swift+https
   * swift+config
   * rbd
   * sheepdog
   * cinder
   * vsphere
   * database
"""))
    ]

    artifact_type_opts = []

    @classmethod
    def list_artifact_type_opts(cls):
        return cls.artifact_type_opts + cls.common_artifact_type_opts

    db_api = artifact_api.ArtifactAPI()

    @classmethod
    def is_blob(cls, field_name):
        """Helper to check that a field is a blob.

        :param field_name: name of the field
        :return: True if the field is a blob, False otherwise
        """
        return isinstance(cls.fields.get(field_name), glare_fields.BlobField)

    @classmethod
    def is_blob_dict(cls, field_name):
        """Helper to check that field is a blob dict.

        :param field_name: name of the field
        :return: True if the field is a blob dict, False otherwise
        """
        return (isinstance(cls.fields.get(field_name), glare_fields.Dict) and
                cls.fields[field_name].element_type ==
                glare_fields.BlobFieldType)

    @classmethod
    def init_artifact(cls, context, values):
        """Initialize an empty versioned object with values.

        Initialize vo object with default values and values specified by user.
        Also reset all changes of initialized object so user can track own
        changes.

        :param context: user context
        :param values: values needs to be set
        :return: artifact with initialized values
        """
        af = cls(context)
        # setup default values for all non specified fields
        default_fields = []
        for field in af.fields:
            if field not in values:
                default_fields.append(field)
        if default_fields:
            af.obj_set_defaults(*default_fields)

        # apply values specified by user
        for name, value in values.items():
            setattr(af, name, value)
        return af

    @classmethod
    def get_type_name(cls):
        """Return type name that allows to find artifact type in Glare

        Type name allows to find artifact type definition in Glare registry.

        :return: string that identifies current artifact type
        """
        raise NotImplementedError()

    def create(self, context):
        """Create new artifact in Glare repo.

        :param context: user context
        :return: created artifact object
        """
        values = self.obj_changes_to_primitive()
        values['type_name'] = self.get_type_name()

        LOG.debug("Sending request to create artifact of type '%(type_name)s'."
                  " New values are %(values)s",
                  {'type_name': self.get_type_name(), 'values': values})

        af_vals = self.db_api.save(context, None, values)
        return self.init_artifact(context, af_vals)

    def save(self, context):
        """Save artifact in Glare repo.

        :param context: user context
        :return: updated artifact object
        """
        values = self.obj_changes_to_primitive()

        LOG.debug("Sending request to update artifact '%(af_id)s'. "
                  "New values are %(values)s",
                  {'af_id': self.id, 'values': values})

        updated_af = self.db_api.save(context, self.id, values)
        return self.init_artifact(context, updated_af)

    @classmethod
    def show(cls, context, artifact_id):
        """Return Artifact from Glare repo

        :param context: user context
        :param artifact_id: id of requested artifact
        :return: requested artifact object
        """
        af = cls.db_api.get(context, artifact_id)
        return cls.init_artifact(context, af)

    @classmethod
    def _get_field_type(cls, obj):
        """Get string representation of field type for filters."""
        if isinstance(obj, fields.IntegerField) or obj is fields.Integer:
            return 'int'
        elif isinstance(obj, fields.FloatField) or obj is fields.Float:
            return 'numeric'
        elif isinstance(obj, fields.FlexibleBooleanField) or \
                obj is fields.FlexibleBoolean:
            return 'bool'
        return 'string'

    @classmethod
    def _parse_sort_values(cls, sort):
        """Prepare sorting parameters for database."""
        new_sort = []
        for key, direction in sort:
            if key not in cls.fields:
                msg = _("The field %s doesn't exist.") % key
                raise exception.BadRequest(msg)
            # check if field can be sorted
            if not cls.fields[key].sortable:
                msg = _("The field %s is not sortable.") % key
                raise exception.BadRequest(msg)
            new_sort.append((key, direction, cls._get_field_type(
                cls.fields.get(key))))
        return new_sort

    @classmethod
    def _validate_filter_ops(cls, filter_name, op):
        field = cls.fields.get(filter_name)
        if op not in field.filter_ops:
            msg = (_("Unsupported filter type '%(key)s'."
                     "The following filters are supported "
                     "%(filters)s") % {
                'key': op, 'filters': str(field.filter_ops)})
            raise exception.BadRequest(message=msg)

    @classmethod
    def _parse_filter_values(cls, filters):
        # input format for filters is list of tuples:
        # (filter_name, filter_value)
        # output format for filters is list of tuples:
        # (field_name, key_name, op, field_type, value)
        new_filters = []

        for filter_name, filter_value in filters:
            if filter_name in ('tags-any', 'tags'):
                if ':' in filter_value:
                    msg = _("Tags are filtered without operator")
                    raise exception.BadRequest(msg)
                new_filters.append(
                    (filter_name, None, None, None, filter_value))
                continue

            key_name = None
            if '.' in filter_name:
                filter_name, key_name = filter_name.rsplit('.', 1)
                if not isinstance(cls.fields.get(filter_name),
                                  glare_fields.Dict):
                    msg = _("Field %s is not Dict") % filter_name
                    raise exception.BadRequest(msg)

            if cls.fields.get(filter_name) is None:
                msg = _("Unable filter '%s'") % filter_name
                raise exception.BadRequest(msg)

            field_type = cls.fields.get(filter_name)
            if isinstance(field_type, glare_fields.List) or isinstance(
                    field_type, glare_fields.Dict) and key_name is not None:
                field_type = field_type.element_type

            try:
                op, val = utils.split_filter_op(filter_value)
                if isinstance(field_type, glare_fields.Dict):
                    if op not in ['eq', 'in']:
                        msg = (_("Unsupported filter type '%s'. The following "
                                 "filters are supported: eq, in") % op)
                        raise exception.BadRequest(message=msg)
                    if op == 'in':
                        new_filters.append((
                            filter_name, utils.split_filter_value_for_quotes(
                                val), op, None, None))
                    else:
                        new_filters.append((
                            filter_name, val, op, None, None))
                else:
                    cls._validate_filter_ops(filter_name, op)
                    if op == 'in':
                        value = [field_type.coerce(cls(), filter_name, value)
                                 for value in
                                 utils.split_filter_value_for_quotes(val)]
                    else:
                        value = field_type.coerce(cls(), filter_name, val)
                    new_filters.append(
                        (filter_name, key_name, op,
                         cls._get_field_type(field_type), value))
            except ValueError:
                msg = _("Invalid filter value: %s") % str(val)
                raise exception.BadRequest(msg)

        return new_filters

    @classmethod
    def list(cls, context, filters=None, marker=None, limit=None,
             sort=None, latest=False):
        """Return list of artifacts requested by user.

        :param context: user context
        :param filters: filters that need to be applied to artifact
        :param marker: the artifact that considered as begin of the list
        so all artifacts before marker (including marker itself) will not be
        added to artifact list
        :param limit: maximum number of items in the list
        :param sort: sorting options
        :param latest: flag that indicates, that only artifacts with highest
        versions should be returned in output
        :return: list of artifact objects
        """

        default_sort_parameters = (
            ('created_at', 'desc', None), ('id', 'asc', None))
        # Parse sort parameters and update them with defaults
        sort = [] if sort is None else cls._parse_sort_values(sort)
        for default_sort in default_sort_parameters:
            for s in sort:
                # If the default sort parameter already in the list - skip it
                if s[0] == default_sort[0]:
                    break
            else:
                sort.append(default_sort)

        default_filter_parameters = [
            ('status', None, 'neq', None, 'deleted')]
        if cls.get_type_name() != 'all':
            default_filter_parameters.append(
                ('type_name', None, 'eq', None, cls.get_type_name()))
        # Parse filter parameters and update them with defaults
        filters = [] if filters is None else cls._parse_filter_values(filters)
        for default_filter in default_filter_parameters:
            if default_filter not in filters:
                filters.append(default_filter)

        return [cls.init_artifact(context, af)
                for af in cls.db_api.list(
                context, filters, marker, limit, sort, latest)]

    @classmethod
    def delete(cls, context, af):
        """Delete artifact and all its blobs from Glare.

        :param context: user context
        :param af: artifact object targeted for deletion
        """
        # marking artifact as deleted
        cls.db_api.save(context, af.id, {'status': 'deleted'})

        # collect all uploaded blobs
        blobs = {}
        for name in af.fields:
            if cls.is_blob(name) or cls.is_blob_dict(name):
                field = getattr(af, name)
                if field:
                    blobs[name] = field

        LOG.debug("Marked artifact %(artifact)s as deleted.",
                  {'artifact': af.id})

        return blobs

    @classmethod
    def get_max_blob_size(cls, field_name):
        """Get the maximum allowed blob size in bytes.

        :param field_name: blob or blob dict field name
        :return: maximum blob size in bytes
        """
        return getattr(cls.fields[field_name], 'max_blob_size')

    @classmethod
    def get_max_folder_size(cls, field_name):
        """Get the maximum allowed folder size in bytes.

        :param field_name: folder (blob dict) field name
        :return: maximum folder size in bytes
        """
        return getattr(cls.fields[field_name], 'max_folder_size')

    @classmethod
    def update_blob(cls, context, af_id, field_name, values):
        """Update blob info in database.

        :param context: user context
        :param af_id: id of modified artifact
        :param field_name: blob or blob dict field name
        :param values: updated blob values
        :return: updated artifact definition in Glare
        """
        af_upd = cls.db_api.update_blob(context, af_id, {field_name: values})
        return cls.init_artifact(context, af_upd)

    # Next comes a collection of hooks for various operations

    @classmethod
    def pre_create_hook(cls, context, af):
        pass

    @classmethod
    def post_create_hook(cls, context, af):
        pass

    @classmethod
    def pre_update_hook(cls, context, af):
        pass

    @classmethod
    def post_update_hook(cls, context, af):
        pass

    @classmethod
    def pre_activate_hook(cls, context, af):
        pass

    @classmethod
    def post_activate_hook(cls, context, af):
        pass

    @classmethod
    def pre_publish_hook(cls, context, af):
        pass

    @classmethod
    def post_publish_hook(cls, context, af):
        pass

    @classmethod
    def pre_deactivate_hook(cls, context, af):
        pass

    @classmethod
    def post_deactivate_hook(cls, context, af):
        pass

    @classmethod
    def pre_reactivate_hook(cls, context, af):
        pass

    @classmethod
    def post_reactivate_hook(cls, context, af):
        pass

    @classmethod
    def pre_upload_hook(cls, context, af, field_name, blob_key, fd):
        return fd

    @classmethod
    def post_upload_hook(cls, context, af, field_name, blob_key):
        pass

    @classmethod
    def pre_add_location_hook(
            cls, context, af, field_name, blob_key, location):
        pass

    @classmethod
    def post_add_location_hook(cls, context, af, field_name, blob_key):
        pass

    @classmethod
    def pre_download_hook(cls, context, af, field_name, blob_key):
        pass

    @classmethod
    def post_download_hook(cls, context, af, field_name, blob_key, fd):
        return fd

    @classmethod
    def pre_delete_hook(cls, context, af):
        pass

    @classmethod
    def post_delete_hook(cls, context, af):
        pass

    def to_notification(self):
        """Return notification body that can be send to listeners.

        :return: dict with notification information
        """
        return {
            'type': self.get_type_name(),
            'id': self.id,
            'description': self.description,
            'name': self.name,
            'version': self.version,
            'visibility': self.visibility,
            'status': self.status,
            'created_at': self.created_at,
            'updated_at': self.updated_at,
            'activated_at': self.activated_at,
            'owner': self.owner
        }

    def to_dict(self):
        """Convert oslo versioned object to dictionary.

        :return: dict with field names and field values
        """
        return self.obj_to_primitive()['versioned_object.data']

    def obj_changes_to_primitive(self):
        changes = self.obj_get_changes()
        res = {}
        for key, val in changes.items():
            if val is not None and hasattr(val, 'to_primitive'):
                res[key] = val.to_primitive()
            else:
                res[key] = val
        return res

    @classmethod
    def _schema_field(cls, field, field_name=''):
        field_type = utils.get_schema_type(field)
        schema = {}

        # generate schema for validators
        for val in getattr(field, 'validators', []):
            schema.update(val.to_jsonschema())

        schema['type'] = (field_type
                          if not field.nullable else [field_type, 'null'])
        schema['glareType'] = utils.get_glare_type(field)
        output_blob_schema = {
            'type': ['object', 'null'],
            'properties': {
                'size': {'type': ['number', 'null']},
                'md5': {'type': ['string', 'null']},
                'sha1': {'type': ['string', 'null']},
                'sha256': {'type': ['string', 'null']},
                'external': {'type': 'boolean'},
                'status': {'type': 'string',
                           'enum': list(
                               glare_fields.BlobFieldType.BLOB_STATUS)},
                'content_type': {'type': 'string'},
            },
            'required': ['size', 'md5', 'sha1', 'sha256', 'external', 'status',
                         'content_type'],
            'additionalProperties': False
        }

        if field.system:
            schema['readOnly'] = True

        if isinstance(field, glare_fields.Dict):
            element_type = utils.get_schema_type(field.element_type)
            property_validators = schema.pop('propertyValidators', [])
            if field.element_type is glare_fields.BlobFieldType:
                schema['additionalProperties'] = output_blob_schema
            else:
                if schema.get('properties'):
                    properties = {}
                    required = schema.pop('required', [])
                    for key in schema.pop('properties'):
                        properties[key] = {
                            'type': (element_type
                                     if key in required
                                     else [element_type, 'null'])}
                        for val in property_validators:
                            properties[key].update(val)
                    schema['properties'] = properties
                    schema['additionalProperties'] = False
                else:
                    schema['additionalProperties'] = {'type': element_type}
                    for val in property_validators:
                        schema['additionalProperties'].update(val)

        if isinstance(field, glare_fields.List):
            items_validators = schema.pop('itemValidators', [])
            schema['items'] = {
                'type': utils.get_schema_type(field.element_type)}
            for val in items_validators:
                schema['items'].update(val)

        if isinstance(field, glare_fields.BlobField):
            schema.update(output_blob_schema)

        if isinstance(field, fields.DateTimeField):
            schema['format'] = 'date-time'

        if field_name == 'status':
            schema['enum'] = cls.STATUS

        if field.description:
            schema['description'] = field.description
        if field.mutable:
            schema['mutable'] = True
        if field.sortable:
            schema['sortable'] = True
        if not field.required_on_activate:
            schema['required_on_activate'] = False
        if field._default is not None:
            schema['default'] = field._default

        schema['filter_ops'] = field.filter_ops

        return schema

    @classmethod
    def gen_schemas(cls):
        """Return json schema representation of the artifact type."""
        schemas_prop = {}
        for field_name, field in cls.fields.items():
            schemas_prop[field_name] = cls._schema_field(
                field, field_name=field_name)
        schemas = {'properties': schemas_prop,
                   'name': cls.get_type_name(),
                   'version': cls.VERSION,
                   'title': 'Artifact type %s of version %s' %
                            (cls.get_type_name(), cls.VERSION),
                   'type': 'object',
                   'required': ['name']}

        return schemas
